/*
 * Constants.cpp: Python bindings for the LLVM compiler infrastructure
 * Copyright (c) 2004, 2005 Jarno Seppänen <firstname.lastname@iki.fi>.
 * Distributed under the Boost Software License, Version 1.0. (See
 * accompanying file LICENSE_1_0.txt or copy at
 * http://www.boost.org/LICENSE_1_0.txt)
 */

#include <boost/python.hpp>
#include <boost/noncopyable.hpp>
#include <llvm/Constants.h>
#include <llvm/DerivedTypes.h>

using namespace boost::python;

namespace llvm { namespace Python {

void wrap_ConstantIntegral()
{
    // Overloaded functions need some C++ preamble
    bool (*classof_ConstantIntegral)(const ConstantIntegral*) = &ConstantIntegral::classof;
    bool (*classof_Value)(const Value*) = &ConstantIntegral::classof;

    class_<ConstantIntegral, boost::noncopyable, bases<Constant> >
        ("ConstantIntegral", no_init)

        .def("getRawValue", &ConstantIntegral::getRawValue)
        .def("isNullValue", &ConstantIntegral::isNullValue)
        .def("isMaxValue", &ConstantIntegral::isMaxValue)
        .def("isMinValue", &ConstantIntegral::isMinValue)
        .def("isAllOnesValue", &ConstantIntegral::isAllOnesValue)
        .def("getMaxValue", &ConstantIntegral::getMaxValue,
             return_value_policy<reference_existing_object>())
        .staticmethod("getMaxValue")
        .def("getMinValue", &ConstantIntegral::getMinValue,
             return_value_policy<reference_existing_object>())
        .staticmethod("getMinValue")
        .def("getAllOnesValue", &ConstantIntegral::getAllOnesValue,
             return_value_policy<reference_existing_object>())
        .staticmethod("getAllOnesValue")
        .def("classof", classof_ConstantIntegral)
        .def("classof", classof_Value)
        .staticmethod("classof")
        ;
}

void wrap_ConstantBool()
{
    // Overloaded functions need some C++ preamble
    ConstantBool* (*get)(bool) = &ConstantBool::get;
    ConstantBool* (*get2)(const Type*, bool) = &ConstantBool::get;
    bool (*classof_ConstantBool)(const ConstantBool*) = &ConstantBool::classof;
    bool (*classof_Value)(const Value*) = &ConstantBool::classof;

    class_<ConstantBool, boost::noncopyable, bases<ConstantIntegral> >
        ("ConstantBool", no_init)

        .add_static_property("True",
                             make_getter(*ConstantBool::True,
                                         return_value_policy<reference_existing_object>()))
        .add_static_property("False",
                             make_getter(*ConstantBool::False,
                                         return_value_policy<reference_existing_object>()))
        .def("get", get,
             return_value_policy<reference_existing_object>())
        .def("get", get2,
             return_value_policy<reference_existing_object>())
        .staticmethod("get")
        .def("inverted", &ConstantBool::inverted,
             return_value_policy<reference_existing_object>())
        .def("getValue", &ConstantBool::getValue)
        .def("isNullValue", &ConstantBool::isNullValue)
        .def("isMaxValue", &ConstantBool::isMaxValue)
        .def("isMinValue", &ConstantBool::isMinValue)
        .def("isAllOnesValue", &ConstantBool::isAllOnesValue)
        .def("classof", classof_ConstantBool)
        .def("classof", classof_Value)
        .staticmethod("classof")
        ;
}

void wrap_ConstantInt()
{
    // Overloaded functions need some C++ preamble
    bool (*classof_ConstantInt)(const ConstantInt*) = &ConstantInt::classof;
    bool (*classof_Value)(const Value*) = &ConstantInt::classof;

    class_<ConstantInt, boost::noncopyable, bases<ConstantIntegral> >
        ("ConstantInt", no_init)

        .def("equalsInt", &ConstantInt::equalsInt)
        .def("get", &ConstantInt::get,
             return_value_policy<reference_existing_object>())
        .staticmethod("get")
        .def("isNullValue", &ConstantInt::isNullValue)
        .def("isMaxValue", &ConstantInt::isMaxValue)
        .def("isMinValue", &ConstantInt::isMinValue)
        .def("classof", classof_ConstantInt)
        .def("classof", classof_Value)
        .staticmethod("classof")
        ;
}

void wrap_ConstantSInt()
{
    // Overloaded functions need some C++ preamble
    bool (*classof_ConstantSInt)(const ConstantSInt*) = &ConstantSInt::classof;
    bool (*classof_Value)(const Value*) = &ConstantSInt::classof;

    class_<ConstantSInt, boost::noncopyable, bases<ConstantInt> >
        ("ConstantSInt", no_init)

        .def("get", &ConstantSInt::get,
             return_value_policy<reference_existing_object>())
        .staticmethod("get")
        .def("isValueValidForType", &ConstantSInt::isValueValidForType)
        .staticmethod("isValueValidForType")
        .def("getValue", &ConstantSInt::getValue)
        .def("isAllOnesValue", &ConstantSInt::isAllOnesValue)
        .def("isMaxValue", &ConstantSInt::isMaxValue)
        .def("isMinValue", &ConstantSInt::isMinValue)
        .def("classof", classof_ConstantSInt)
        .def("classof", classof_Value)
        .staticmethod("classof")
        ;
}

void wrap_ConstantUInt()
{
    // Overloaded functions need some C++ preamble
    bool (*classof_ConstantUInt)(const ConstantUInt*) = &ConstantUInt::classof;
    bool (*classof_Value)(const Value*) = &ConstantUInt::classof;

    class_<ConstantUInt, boost::noncopyable, bases<ConstantInt> >
        ("ConstantUInt", no_init)

        .def("get", &ConstantUInt::get,
             return_value_policy<reference_existing_object>())
        .staticmethod("get")
        .def("isValueValidForType", &ConstantUInt::isValueValidForType)
        .staticmethod("isValueValidForType")
        .def("getValue", &ConstantUInt::getValue)
        .def("isAllOnesValue", &ConstantUInt::isAllOnesValue)
        .def("isMaxValue", &ConstantUInt::isMaxValue)
        .def("isMinValue", &ConstantUInt::isMinValue)
        .def("classof", classof_ConstantUInt)
        .def("classof", classof_Value)
        .staticmethod("classof")
        ;
}

void wrap_ConstantFP()
{
    // Overloaded functions need some C++ preamble
    bool (*classof_ConstantFP)(const ConstantFP*) = &ConstantFP::classof;
    bool (*classof_Value)(const Value*) = &ConstantFP::classof;

    class_<ConstantFP, boost::noncopyable, bases<Constant> >
        ("ConstantFP", no_init)

        .def("get", &ConstantFP::get,
             return_value_policy<reference_existing_object>())
        .staticmethod("get")
        .def("isValueValidForType", &ConstantFP::isValueValidForType)
        .staticmethod("isValueValidForType")
        .def("getValue", &ConstantFP::getValue)
        .def("isNullValue", &ConstantFP::isNullValue)
        .def("isExactlyValue", &ConstantFP::isExactlyValue)
        .def("classof", classof_ConstantFP)
        .def("classof", classof_Value)
        .staticmethod("classof")
        ;
}

void wrap_ConstantAggregateZero()
{
    // Overloaded functions need some C++ preamble
    bool (*classof_ConstantAggregateZero)(const ConstantAggregateZero*) = &ConstantAggregateZero::classof;
    bool (*classof_Value)(const Value*) = &ConstantAggregateZero::classof;

    class_<ConstantAggregateZero, boost::noncopyable, bases<Constant> >
        ("ConstantAggregateZero", no_init)

        .def("get", &ConstantAggregateZero::get,
             return_value_policy<reference_existing_object>())
        .staticmethod("get")
        .def("isNullValue", &ConstantAggregateZero::isNullValue)
        .def("destroyConstant", &ConstantAggregateZero::destroyConstant)
        .def("replaceUsesOfWithOnConstant", &ConstantAggregateZero::replaceUsesOfWithOnConstant)
        .def("classof", classof_ConstantAggregateZero)
        .def("classof", classof_Value)
        .staticmethod("classof")
        ;
}

void wrap_ConstantArray()
{
    // Overloaded functions need some C++ preamble
    Constant* (*get)(const ArrayType*, const std::vector<Constant*>&) = &ConstantArray::get;
    Constant* (*get2)(const std::string&) = &ConstantArray::get;
    bool (*classof_ConstantArray)(const ConstantArray*) = &ConstantArray::classof;
    bool (*classof_Value)(const Value*) = &ConstantArray::classof;

    class_<ConstantArray, boost::noncopyable, bases<Constant> >
        ("ConstantArray", no_init)

        .def("get", get,
             return_value_policy<reference_existing_object>())
        .def("get", get2,
             return_value_policy<reference_existing_object>())
        .staticmethod("get")
        .def("getType", &ConstantArray::getType,
             return_value_policy<reference_existing_object>())
        .def("isString", &ConstantArray::isString)
        .def("getAsString", &ConstantArray::getAsString)
        .def("isNullValue", &ConstantArray::isNullValue)
        .def("destroyConstant", &ConstantArray::destroyConstant)
        .def("replaceUsesOfWithOnConstant", &ConstantArray::replaceUsesOfWithOnConstant)
        .def("classof", classof_ConstantArray)
        .def("classof", classof_Value)
        .staticmethod("classof")
        ;
}

void wrap_ConstantStruct()
{
    // Overloaded functions need some C++ preamble
    Constant* (*get)(const StructType*, const std::vector<Constant*>&) = &ConstantStruct::get;
    Constant* (*get2)(const std::vector<Constant*>&) = &ConstantStruct::get;
    bool (*classof_ConstantStruct)(const ConstantStruct*) = &ConstantStruct::classof;
    bool (*classof_Value)(const Value*) = &ConstantStruct::classof;

    class_<ConstantStruct, boost::noncopyable, bases<Constant> >
        ("ConstantStruct", no_init)

        .def("get", get,
             return_value_policy<reference_existing_object>())
        .def("get", get2,
             return_value_policy<reference_existing_object>())
        .staticmethod("get")
        .def("getType", &ConstantStruct::getType,
             return_value_policy<reference_existing_object>())
        .def("isNullValue", &ConstantStruct::isNullValue)
        .def("destroyConstant", &ConstantStruct::destroyConstant)
        .def("replaceUsesOfWithOnConstant", &ConstantStruct::replaceUsesOfWithOnConstant)
        .def("classof", classof_ConstantStruct)
        .def("classof", classof_Value)
        .staticmethod("classof")
        ;
}

void wrap_ConstantPacked()
{
    // Overloaded functions need some C++ preamble
    Constant* (*get)(const PackedType*, const std::vector<Constant*>&) = &ConstantPacked::get;
    Constant* (*get2)(const std::vector<Constant*>&) = &ConstantPacked::get;
    bool (*classof_ConstantPacked)(const ConstantPacked*) = &ConstantPacked::classof;
    bool (*classof_Value)(const Value*) = &ConstantPacked::classof;

    class_<ConstantPacked, boost::noncopyable, bases<Constant> >
        ("ConstantPacked", no_init)

        .def("get", get,
             return_value_policy<reference_existing_object>())
        .def("get", get2,
             return_value_policy<reference_existing_object>())
        .staticmethod("get")
        .def("getType", &ConstantPacked::getType,
             return_value_policy<reference_existing_object>())
        .def("isNullValue", &ConstantPacked::isNullValue)
        .def("destroyConstant", &ConstantPacked::destroyConstant)
        .def("replaceUsesOfWithOnConstant", &ConstantPacked::replaceUsesOfWithOnConstant)
        .def("classof", classof_ConstantPacked)
        .def("classof", classof_Value)
        .staticmethod("classof")
        ;
}

void wrap_ConstantPointerNull()
{
    // Overloaded functions need some C++ preamble
    bool (*classof_ConstantPointerNull)(const ConstantPointerNull*) = &ConstantPointerNull::classof;
    bool (*classof_Value)(const Value*) = &ConstantPointerNull::classof;

    class_<ConstantPointerNull, boost::noncopyable, bases<Constant> >
        ("ConstantPointerNull", no_init)

        .def("get", &ConstantPointerNull::get,
             return_value_policy<reference_existing_object>())
        .staticmethod("get")
        .def("isNullValue", &ConstantPointerNull::isNullValue)
        .def("destroyConstant", &ConstantPointerNull::destroyConstant)
        .def("getType", &ConstantPointerNull::getType,
             return_value_policy<reference_existing_object>())
        .def("classof", classof_ConstantPointerNull)
        .def("classof", classof_Value)
        .staticmethod("classof")
        ;
}

void wrap_ConstantExpr()
{
    // Overloaded functions need some C++ preamble
    Constant* (*getGetElementPtr)(Constant*, const std::vector<Constant*>&) = &ConstantExpr::getGetElementPtr;
    Constant* (*getGetElementPtr2)(Constant*, const std::vector<Value*>&) = &ConstantExpr::getGetElementPtr;
    Constant* (ConstantExpr::*getOperand)(unsigned) = &ConstantExpr::getOperand;
    Constant* (ConstantExpr::*getOperand2)(unsigned) const = &ConstantExpr::getOperand;
    bool (*classof_ConstantExpr)(const ConstantExpr*) = &ConstantExpr::classof;
    bool (*classof_Value)(const Value*) = &ConstantExpr::classof;

    class_<ConstantExpr, boost::noncopyable, bases<Constant> >
        ("ConstantExpr", no_init)

        .def("getCast", &ConstantExpr::getCast,
             return_value_policy<reference_existing_object>())
        .staticmethod("getCast")
        .def("getSignExtend", &ConstantExpr::getSignExtend,
             return_value_policy<reference_existing_object>())
        .staticmethod("getSignExtend")
        .def("getZeroExtend", &ConstantExpr::getZeroExtend,
             return_value_policy<reference_existing_object>())
        .staticmethod("getZeroExtend")
        .def("getSelect", &ConstantExpr::getSelect,
             return_value_policy<reference_existing_object>())
        .staticmethod("getSelect")
        .def("getSizeOf", &ConstantExpr::getSizeOf,
             return_value_policy<reference_existing_object>())
        .staticmethod("getSizeOf")
        .def("get", &ConstantExpr::get,
             return_value_policy<reference_existing_object>())
        .staticmethod("get")
        .def("getNeg", &ConstantExpr::getNeg,
             return_value_policy<reference_existing_object>())
        .staticmethod("getNeg")
        .def("getNot", &ConstantExpr::getNot,
             return_value_policy<reference_existing_object>())
        .staticmethod("getNot")
        .def("getAdd", &ConstantExpr::getAdd,
             return_value_policy<reference_existing_object>())
        .staticmethod("getAdd")
        .def("getSub", &ConstantExpr::getSub,
             return_value_policy<reference_existing_object>())
        .staticmethod("getSub")
        .def("getMul", &ConstantExpr::getMul,
             return_value_policy<reference_existing_object>())
        .staticmethod("getMul")
        .def("getDiv", &ConstantExpr::getDiv,
             return_value_policy<reference_existing_object>())
        .staticmethod("getDiv")
        .def("getRem", &ConstantExpr::getRem,
             return_value_policy<reference_existing_object>())
        .staticmethod("getRem")
        .def("getAnd", &ConstantExpr::getAnd,
             return_value_policy<reference_existing_object>())
        .staticmethod("getAnd")
        .def("getOr", &ConstantExpr::getOr,
             return_value_policy<reference_existing_object>())
        .staticmethod("getOr")
        .def("getXor", &ConstantExpr::getXor,
             return_value_policy<reference_existing_object>())
        .staticmethod("getXor")
        .def("getSetEQ", &ConstantExpr::getSetEQ,
             return_value_policy<reference_existing_object>())
        .staticmethod("getSetEQ")
        .def("getSetNE", &ConstantExpr::getSetNE,
             return_value_policy<reference_existing_object>())
        .staticmethod("getSetNE")
        .def("getSetLT", &ConstantExpr::getSetLT,
             return_value_policy<reference_existing_object>())
        .staticmethod("getSetLT")
        .def("getSetGT", &ConstantExpr::getSetGT,
             return_value_policy<reference_existing_object>())
        .staticmethod("getSetGT")
        .def("getSetLE", &ConstantExpr::getSetLE,
             return_value_policy<reference_existing_object>())
        .staticmethod("getSetLE")
        .def("getSetGE", &ConstantExpr::getSetGE,
             return_value_policy<reference_existing_object>())
        .staticmethod("getSetGE")
        .def("getShl", &ConstantExpr::getShl,
             return_value_policy<reference_existing_object>())
        .staticmethod("getShl")
        .def("getShr", &ConstantExpr::getShr,
             return_value_policy<reference_existing_object>())
        .staticmethod("getShr")
        .def("getUShr", &ConstantExpr::getUShr,
             return_value_policy<reference_existing_object>())
        .staticmethod("getUShr")
        .def("getSShr", &ConstantExpr::getSShr,
             return_value_policy<reference_existing_object>())
        .staticmethod("getSShr")
        .def("getGetElementPtr", getGetElementPtr,
             return_value_policy<reference_existing_object>())
        .def("getGetElementPtr", getGetElementPtr2,
             return_value_policy<reference_existing_object>())
        .staticmethod("getGetElementPtr")
        .def("isNullValue", &ConstantExpr::isNullValue)
        .def("getOpcode", &ConstantExpr::getOpcode)
        .def("getOpcodeName", &ConstantExpr::getOpcodeName)
        .def("destroyConstant", &ConstantExpr::destroyConstant)
        .def("replaceUsesOfWithOnConstant", &ConstantExpr::replaceUsesOfWithOnConstant)
        .def("getOperand", getOperand,
             return_value_policy<reference_existing_object>())
        .def("getOperand", getOperand2,
             return_value_policy<reference_existing_object>())
        .def("classof", classof_ConstantExpr)
        .def("classof", classof_Value)
        .staticmethod("classof")
        ;
}

void wrap_UndefValue()
{
    // Overloaded functions need some C++ preamble
    bool (*classof_UndefValue)(const UndefValue*) = &UndefValue::classof;
    bool (*classof_Value)(const Value*) = &UndefValue::classof;

    class_<UndefValue, boost::noncopyable, bases<Constant> >
        ("UndefValue", no_init)

        .def("get", &UndefValue::get,
             return_value_policy<reference_existing_object>())
        .staticmethod("get")
        .def("isNullValue", &UndefValue::isNullValue)
        .def("destroyConstant", &UndefValue::destroyConstant)
        .def("classof", classof_UndefValue)
        .def("classof", classof_Value)
        .staticmethod("classof")
        ;
}

void wrap_Constants()
{
    wrap_ConstantIntegral();
    wrap_ConstantBool();
    wrap_ConstantInt();
    wrap_ConstantSInt();
    wrap_ConstantUInt();
    wrap_ConstantFP();
    wrap_ConstantAggregateZero();
    wrap_ConstantArray();
    wrap_ConstantStruct();
    wrap_ConstantPacked();
    wrap_ConstantPointerNull();
    wrap_ConstantExpr();
    wrap_UndefValue();
}

}} // namespace llvm::Python

// EOF
