/*
 * DerivedTypes.cpp: Python bindings for the LLVM compiler infrastructure
 * Copyright (c) 2004, 2005 Jarno Seppänen <firstname.lastname@iki.fi>.
 * Distributed under the Boost Software License, Version 1.0. (See
 * accompanying file LICENSE_1_0.txt or copy at
 * http://www.boost.org/LICENSE_1_0.txt)
 */

#include <boost/python.hpp>
#include <boost/noncopyable.hpp>
#include <llvm/DerivedTypes.h>
#include <llvm/Value.h>

using namespace boost::python;

namespace llvm { namespace Python {

void wrap_DerivedType()
{
    // Overloaded functions need some C++ preamble
    bool (*classof_DerivedType)(const DerivedType*) = &DerivedType::classof;
    bool (*classof_Type)(const Type*) = &DerivedType::classof;

    class_<DerivedType, boost::noncopyable, bases<Type, AbstractTypeUser> >("DerivedType", no_init)

        .def("addAbstractTypeUser", &DerivedType::addAbstractTypeUser)
        .def("removeAbstractTypeUser", &DerivedType::removeAbstractTypeUser)
        .def("refineAbstractTypeTo", &DerivedType::refineAbstractTypeTo)
        .def("dump", &DerivedType::dump)
        .def("classof", classof_DerivedType)
        .def("classof", classof_Type)
        .staticmethod("classof")
        ;
}

void wrap_FunctionType()
{
    // Overloaded functions need some C++ preamble
    bool (*classof_FunctionType)(const FunctionType*) = &FunctionType::classof;
    bool (*classof_Type)(const Type*) = &FunctionType::classof;

    class_<FunctionType, boost::noncopyable, bases<DerivedType> >("FunctionType", no_init)

        .def("get", &FunctionType::get,
             return_value_policy<manage_new_object>())
        .staticmethod("get")
        .def("isVarArg", &FunctionType::isVarArg)
        .def("getReturnType", &FunctionType::getReturnType,
             return_internal_reference<>())
        .add_property("params", range(&FunctionType::param_begin, &FunctionType::param_end))
        //.def("param_begin", &FunctionType::param_begin)
        //.def("param_end", &FunctionType::param_end)
        .def("getParamType", &FunctionType::getParamType,
             return_internal_reference<>())
        .def("getNumParams", &FunctionType::getNumParams)
        .def("refineAbstractType", &FunctionType::refineAbstractType)
        .def("typeBecameConcrete", &FunctionType::typeBecameConcrete)
        .def("classof", classof_FunctionType)
        .def("classof", classof_Type)
        .staticmethod("classof")
        ;
}

void wrap_CompositeType()
{
    // Overloaded functions need some C++ preamble
    bool (*classof_CompositeType)(const CompositeType*) = &CompositeType::classof;
    bool (*classof_Type)(const Type*) = &CompositeType::classof;

    class_<CompositeType, boost::noncopyable, bases<DerivedType> >("CompositeType", no_init)

        .def("getTypeAtIndex", &CompositeType::getTypeAtIndex,
             return_internal_reference<>())
        .def("indexValue", &CompositeType::indexValid)
        .def("classof", classof_CompositeType)
        .def("classof", classof_Type)
        .staticmethod("classof")
        ;
}

void wrap_StructType()
{
    // Overloaded functions need some C++ preamble
    bool (*classof_StructType)(const StructType*) = &StructType::classof;
    bool (*classof_Type)(const Type*) = &StructType::classof;

    class_<StructType, boost::noncopyable, bases<CompositeType> >("StructType", no_init)

        .def("get", &StructType::get,
             return_value_policy<manage_new_object>())
        .staticmethod("get")
        .add_property("elements", range(&StructType::element_begin, &StructType::element_end))
        //.def("element_begin", &StructType::element_begin)
        //.def("element_end", &StructType::element_end)
        .def("getNumElements", &StructType::getNumElements)
        .def("getElementType", &StructType::getElementType,
             return_internal_reference<>())
        .def("getTypeAtIndex", &StructType::getTypeAtIndex,
             return_internal_reference<>())
        .def("indexValid", &StructType::indexValid)
        .def("refineAbstractType", &StructType::refineAbstractType)
        .def("typeBecameConcrete", &StructType::typeBecameConcrete)
        .def("classof", classof_StructType)
        .def("classof", classof_Type)
        .staticmethod("classof")
        ;
}

void wrap_SequentialType()
{
    // Overloaded functions need some C++ preamble
    bool (*classof_SequentialType)(const SequentialType*) = &SequentialType::classof;
    bool (*classof_Type)(const Type*) = &SequentialType::classof;

    class_<SequentialType, boost::noncopyable, bases<CompositeType> >("SequentialType", no_init)

        .def("getElementType", &SequentialType::getElementType,
             return_internal_reference<>())
        .def("indexValid", &SequentialType::indexValid)
        .def("getTypeAtIndex", &SequentialType::getTypeAtIndex,
             return_internal_reference<>())
        .def("classof", classof_SequentialType)
        .def("classof", classof_Type)
        .staticmethod("classof")
        ;
}

void wrap_ArrayType()
{
    // Overloaded functions need some C++ preamble
    bool (*classof_ArrayType)(const ArrayType*) = &ArrayType::classof;
    bool (*classof_Type)(const Type*) = &ArrayType::classof;

    class_<ArrayType, boost::noncopyable, bases<SequentialType> >("ArrayType", no_init)

        .def("get", &ArrayType::get,
             return_value_policy<manage_new_object>())
        .staticmethod("get")
        .def("getNumElements", &ArrayType::getNumElements)
        .def("refineAbstractType", &ArrayType::refineAbstractType)
        .def("typeBecameConcrete", &ArrayType::typeBecameConcrete)
        .def("classof", classof_ArrayType)
        .def("classof", classof_Type)
        .staticmethod("classof")
        ;
}

void wrap_PackedType()
{
    // Overloaded functions need some C++ preamble
    bool (*classof_PackedType)(const PackedType*) = &PackedType::classof;
    bool (*classof_Type)(const Type*) = &PackedType::classof;

    class_<PackedType, boost::noncopyable, bases<SequentialType> >("PackedType", no_init)

        .def("get", &PackedType::get,
             return_value_policy<manage_new_object>())
        .staticmethod("get")
        .def("getNumElements", &PackedType::getNumElements)
        .def("refineAbstractType", &PackedType::refineAbstractType)
        .def("typeBecameConcrete", &PackedType::typeBecameConcrete)
        .def("classof", classof_PackedType)
        .def("classof", classof_Type)
        .staticmethod("classof")
        ;
}

void wrap_PointerType()
{
    // Overloaded functions need some C++ preamble
    bool (*classof_PointerType)(const PointerType*) = &PointerType::classof;
    bool (*classof_Type)(const Type*) = &PointerType::classof;

    class_<PointerType, boost::noncopyable, bases<SequentialType> >("PointerType", no_init)

        .def("get", &PointerType::get,
             return_value_policy<manage_new_object>())
        .staticmethod("get")
        .def("refineAbstractType", &PointerType::refineAbstractType)
        .def("typeBecameConcrete", &PointerType::typeBecameConcrete)
        .def("classof", classof_PointerType)
        .def("classof", classof_Type)
        .staticmethod("classof")
        ;
}

void wrap_OpaqueType()
{
    // Overloaded functions need some C++ preamble
    bool (*classof_OpaqueType)(const OpaqueType*) = &OpaqueType::classof;
    bool (*classof_Type)(const Type*) = &OpaqueType::classof;

    class_<OpaqueType, boost::noncopyable, bases<DerivedType> >("OpaqueType", no_init)

        .def("get", &OpaqueType::get,
             return_value_policy<manage_new_object>())
        .staticmethod("get")
        .def("refineAbstractType", &OpaqueType::refineAbstractType)
        .def("typeBecameConcrete", &OpaqueType::typeBecameConcrete)
        .def("classof", classof_OpaqueType)
        .def("classof", classof_Type)
        .staticmethod("classof")
        ;
}

void wrap_DerivedTypes()
{
    wrap_DerivedType();
    wrap_FunctionType();
    wrap_CompositeType();
    wrap_StructType();
    wrap_SequentialType();
    wrap_ArrayType();
    wrap_PackedType();
    wrap_PointerType();
    wrap_OpaqueType();
}

}} // namespace llvm::Python

// EOF
