/*
 * Function.cpp: Python bindings for the LLVM compiler infrastructure
 * Copyright (c) 2004, 2005 Jarno Seppänen <firstname.lastname@iki.fi>.
 * Distributed under the Boost Software License, Version 1.0. (See
 * accompanying file LICENSE_1_0.txt or copy at
 * http://www.boost.org/LICENSE_1_0.txt)
 */

#include <boost/python.hpp>
#include <boost/noncopyable.hpp>
#include <llvm/Assembly/AsmAnnotationWriter.h>
#include <llvm/DerivedTypes.h>
#include <llvm/Function.h>
#include <llvm/Module.h>
#include <llvm/SymbolTable.h>

#include "scitbx/container_conversions.h"

using namespace boost::python;
using namespace scitbx::boost_python::container_conversions;

#include "inst_init.h"

namespace llvm { namespace Python {

/** Python __str__ wrapper support function. */
std::ostream& operator<<(std::ostream& OS, const Function& F)
{
    F.print(OS);
    return OS;
}

template Function* InstInitFn4(const FunctionType*, GlobalValue::LinkageTypes, std::string, Module*);

void wrap_Function()
{
    // Name C++ overloads
    const Function* (Function::*getNext)() const = &Function::getNext;
    const Function* (Function::*getPrev)() const = &Function::getPrev;
    const Function::ArgumentListType& (Function::*getArgumentList)() const = &Function::getArgumentList;
    const Function::BasicBlockListType& (Function::*getBasicBlockList)() const = &Function::getBasicBlockList;
    const BasicBlock& (Function::*getEntryBlock)() const = &Function::getEntryBlock;
    const SymbolTable& (Function::*getSymbolTable)() const = &Function::getSymbolTable;

    Function::const_iterator (Function::*begin)() const = &Function::begin;
    Function::const_iterator (Function::*end)() const = &Function::end;
    Function::const_reverse_iterator (Function::*rbegin)() const = &Function::rbegin;
    Function::const_reverse_iterator (Function::*rend)() const = &Function::rend;
    const BasicBlock& (Function::*front)() const = &Function::front;
    const BasicBlock& (Function::*back)() const = &Function::back;

    Function::const_aiterator (Function::*abegin)() const = &Function::abegin;
    Function::const_aiterator (Function::*aend)() const = &Function::aend;
    Function::const_reverse_aiterator (Function::*arbegin)() const = &Function::arbegin;
    Function::const_reverse_aiterator (Function::*arend)() const = &Function::arend;
    const Argument& (Function::*afront)() const = &Function::afront;
    const Argument& (Function::*aback)() const = &Function::aback;

    void (Function::*print)(std::ostream&) const = &Function::print;
    void (Function::*print_AAW)(std::ostream&, AssemblyAnnotationWriter*) const = &Function::print;
    bool (*classof_Function)(const Function*) = &Function::classof;
    bool (*classof_Value)(const Value*) = &Function::classof;

    class_<Function, boost::noncopyable, bases<GlobalValue, Annotable> >
        ("Function",
         init<const FunctionType*, GlobalValue::LinkageTypes, optional<std::string> >())
        // ownership transmits to Module*
        .def("init",
             InstInitFn4<Function, const FunctionType*, GlobalValue::LinkageTypes, std::string, Module*>,
             return_value_policy<reference_existing_object,
                 with_custodian_and_ward_postcall<4, 0> >())
        .staticmethod("init")

        .def("setName", &Function::setName)
        .def("getReturnType", &Function::getReturnType,
             return_internal_reference<>())
        .def("getFunctionType", &Function::getFunctionType,
             return_internal_reference<>())
        .def("isVarArg", &Function::isVarArg)
        .def("isExternal", &Function::isExternal)
        .def("getIntrinsicID", &Function::getIntrinsicID)
        .def("isIntrinsic", &Function::isIntrinsic)
        .def("renameLocalSymbols", &Function::renameLocalSymbols)
        .def("deleteBody", &Function::deleteBody)
        .def("removeFromParent", &Function::removeFromParent)
        .def("eraseFromParent", &Function::eraseFromParent)
        .def("getNext", getNext,
             return_internal_reference<>())
        .def("getPrev", getPrev,
             return_internal_reference<>())
        .def("getArgumentList", getArgumentList,
             return_internal_reference<>())
        .def("getBasicBlockList", getBasicBlockList,
             return_internal_reference<>())
        .def("getEntryBlock", getEntryBlock,
             return_internal_reference<>())
        .def("getSymbolTable", getSymbolTable,
             return_internal_reference<>())
        .add_property("basic_blocks", range<return_internal_reference<> >(begin, end))
        //.def("begin", begin)
        //.def("end", end)
        .add_property("reverse_basic_blocks", range<return_internal_reference<> >(rbegin, rend))
        //.def("rbegin", rbegin)
        //.def("rend", rend)
        .def("size", &Function::size)
        .def("empty", &Function::empty)
        .def("front", front,
             return_internal_reference<>())
        .def("back", back,
             return_internal_reference<>())
        .add_property("arguments", range<return_internal_reference<> >(abegin, aend))
        //.def("abegin", abegin)
        //.def("aend", aend)
        .add_property("reverse_arguments", range<return_internal_reference<> >(arbegin, arend))
        //.def("arbegin", arbegin)
        //.def("arend", arend)
        .def("asize", &Function::asize)
        .def("aempty", &Function::aempty)
        .def("afront", afront,
             return_internal_reference<>())
        .def("aback", aback,
             return_internal_reference<>())
        .def("print_", print)
        .def("print_", print_AAW)
        //FIXME error: no matching function for call to `boost::python::api::object::visit(...)
        //.def(str(self))
        .def("viewCFG", &Function::viewCFG)
        .def("viewCFGOnly", &Function::viewCFGOnly)
        .def("classof", classof_Function)
        .def("classof", classof_Value)
        .staticmethod("classof")
        .def("dropAllReferences", &Function::dropAllReferences)
        ;

    // Register conversions for tuple return values
    to_python_converter<iplist<Argument>,
                        to_tuple<iplist<Argument> > >();
    to_python_converter<iplist<BasicBlock>,
                        to_tuple<iplist<BasicBlock> > >();
}
}} // namespace llvm::Python

// EOF
