/*
 * Instruction.cpp: Python bindings for the LLVM compiler infrastructure
 * Copyright (c) 2004, 2005 Jarno Seppänen <firstname.lastname@iki.fi>.
 * Distributed under the Boost Software License, Version 1.0. (See
 * accompanying file LICENSE_1_0.txt or copy at
 * http://www.boost.org/LICENSE_1_0.txt)
 */

#include <boost/python.hpp>
#include <boost/noncopyable.hpp>
#include <llvm/Assembly/AsmAnnotationWriter.h>
#include <llvm/BasicBlock.h>
#include <llvm/Instruction.h>
#include <llvm/SymbolTable.h>
#include <llvm/Type.h>

using namespace boost::python;

namespace llvm { namespace Python {

/** Python __str__ wrapper support function. */
std::ostream& operator<<(std::ostream& OS, const Instruction& I)
{
    I.print(OS);
    return OS;
}

void wrap_Instruction()
{
    // Name C++ overloads
    const BasicBlock* (Instruction::*getParent)() const = &Instruction::getParent;
    const Instruction* (Instruction::*getNext)() const = &Instruction::getNext;
    const Instruction* (Instruction::*getPrev)() const = &Instruction::getPrev;
    const char* (Instruction::*getOpcodeName)() const = &Instruction::getOpcodeName;
    const char* (*getOpcodeName_static)(unsigned) = &Instruction::getOpcodeName;
    bool (*isTerminator_static)(unsigned) = &Instruction::isTerminator;
    bool (Instruction::*isTerminator)() const = &Instruction::isTerminator;
    bool (Instruction::*isAssociative)() const = &Instruction::isAssociative;
    bool (*isAssociative_static)(unsigned, const Type*) = &Instruction::isAssociative;
    bool (Instruction::*isCommutative)() const = &Instruction::isCommutative;
    bool (*isCommutative_static)(unsigned) = &Instruction::isCommutative;
    bool (Instruction::*isRelational)() const = &Instruction::isRelational;
    bool (*isRelational_static)(unsigned) = &Instruction::isRelational;
    bool (Instruction::*isTrapping)() const = &Instruction::isTrapping;
    bool (*isTrapping_static)(unsigned) = &Instruction::isTrapping;
    void (Instruction::*print)(std::ostream&) const = &Instruction::print;
    void (Instruction::*print_AAW)(std::ostream&, AssemblyAnnotationWriter*) const = &Instruction::print;
    bool (*classof_Instruction)(const Instruction*) = &Instruction::classof;
    bool (*classof_Value)(const Value*) = &Instruction::classof;

    scope InstructionScope
        = class_<Instruction, boost::noncopyable, bases<User> >("Instruction", no_init)

        .def("setName", &Instruction::setName)
        .def("mayWriteToMemory", &Instruction::mayWriteToMemory)
        .def("clone", &Instruction::clone,
             return_value_policy<manage_new_object>())
        .def("isIdenticalTo", &Instruction::isIdenticalTo)
        .def("getParent", getParent,
             return_internal_reference<>())
        .def("getNext", getNext,
             return_internal_reference<>())
        .def("getPrev", getPrev,
             return_internal_reference<>())
        .def("removeFromParent", &Instruction::removeFromParent)
        .def("eraseFromParent", &Instruction::eraseFromParent)
        .def("getOpcode", &Instruction::getOpcode)
        .def("getOpcodeName", getOpcodeName)
        .def("getOpcodeName_Opcode", getOpcodeName_static)
        .staticmethod("getOpcodeName_Opcode")
        .def("isTerminator", isTerminator)
        .def("isTerminator_Opcode", isTerminator_static)
        .staticmethod("isTerminator_Opcode")
        .def("isBinaryOp", &Instruction::isBinaryOp)
        .def("isAssociative", isAssociative)
        .def("isAssociative_Opcode", isAssociative_static)
        .staticmethod("isAssociative_Opcode")
        .def("isCommutative", isCommutative)
        .def("isCommutative_Opcode", isCommutative_static)
        .staticmethod("isCommutative_Opcode")
        .def("isRelational", isRelational)
        .def("isRelational_Opcode", isRelational_static)
        .staticmethod("isRelational_Opcode")
        .def("isTrapping", isTrapping)
        .def("isTrapping_Opcode", isTrapping_static)
        .staticmethod("isTrapping_Opcode")
        .def("print_", print)
        .def("print_", print_AAW)
        //FIXME error: no matching function for call to `boost::python::api::object::visit(...)
        //.def(str(self))
        .def("classof", classof_Instruction)
        .def("classof", classof_Value)
        .staticmethod("classof")
        ;

    enum_<Instruction::TermOps>("TermOps")
        .value("TermOpsBegin", Instruction::TermOpsBegin)
        .value("Ret", Instruction::Ret)
        .value("Br", Instruction::Br)
        .value("Switch", Instruction::Switch)
        .value("Invoke", Instruction::Invoke)
        .value("Unwind", Instruction::Unwind)
        .value("Unreachable", Instruction::Unreachable)
        .value("TermOpsEnd", Instruction::TermOpsEnd)
        ;

    enum_<Instruction::BinaryOps>("BinaryOps")
        .value("Add", Instruction::Add)
        .value("Sub", Instruction::Sub)
        .value("Mul", Instruction::Mul)
        .value("Div", Instruction::Div)
        .value("Rem", Instruction::Rem)
        .value("And", Instruction::And)
        .value("Or", Instruction::Or)
        .value("Xor", Instruction::Xor)
        .value("SetEQ", Instruction::SetEQ)
        .value("SetNE", Instruction::SetNE)
        .value("SetLE", Instruction::SetLE)
        .value("SetGE", Instruction::SetGE)
        .value("SetLT", Instruction::SetLT)
        .value("SetGT", Instruction::SetGT)
        ;

    enum_<Instruction::MemoryOps>("MemoryOps")
        .value("Malloc", Instruction::Malloc)
        .value("Free", Instruction::Free)
        .value("Alloca", Instruction::Alloca)
        .value("Load", Instruction::Load)
        .value("Store", Instruction::Store)
        .value("GetElementPtr", Instruction::GetElementPtr)
        ;

    enum_<Instruction::OtherOps>("OtherOps")
        .value("PHI", Instruction::PHI)
        .value("Cast", Instruction::Cast)
        .value("Call", Instruction::Call)
        .value("Shl", Instruction::Shl)
        .value("Shr", Instruction::Shr)
        .value("VANext", Instruction::VANext)
        .value("VAArg", Instruction::VAArg)
        .value("Select", Instruction::Select)
        .value("UserOp1", Instruction::UserOp1)
        .value("UserOp2", Instruction::UserOp2)
        ;
}

}} // namespace llvm::Python

// EOF

