/*
 * Instructions.cpp: Python bindings for the LLVM compiler infrastructure
 * Copyright (c) 2004, 2005 Jarno Seppänen <firstname.lastname@iki.fi>.
 * Distributed under the Boost Software License, Version 1.0. (See
 * accompanying file LICENSE_1_0.txt or copy at
 * http://www.boost.org/LICENSE_1_0.txt)
 */

#include <boost/python.hpp>
#include <boost/noncopyable.hpp>
#include <llvm/BasicBlock.h>
#include <llvm/Constant.h>
#include <llvm/DerivedTypes.h>
#include <llvm/Function.h>
#include <llvm/Instructions.h>
#include <llvm/Type.h>
#include <llvm/Value.h>

using namespace boost::python;

// Instruction constructor helpers
#include "inst_init.h"

namespace llvm { namespace Python {

void wrap_AllocationInst()
{
    // Name C++ overloads
    const Value* (AllocationInst::*getArraySize)() const = &AllocationInst::getArraySize;
    bool (*classof_AllocationInst)(const AllocationInst*) = &AllocationInst::classof;
    bool (*classof_Instruction)(const Instruction*) = &AllocationInst::classof;
    bool (*classof_Value)(const Value*) = &AllocationInst::classof;

    class_<AllocationInst, boost::noncopyable, bases<Instruction> >
      ("AllocationInst", no_init)
        .def("isArrayAllocation", &AllocationInst::isArrayAllocation)
        .def("getArraySize", getArraySize,
             return_internal_reference<>())
        .def("getType", &AllocationInst::getType,
             return_internal_reference<>())
        .def("getAllocatedType", &AllocationInst::getAllocatedType,
             return_internal_reference<>())
        .def("clone", &AllocationInst::clone,
             return_value_policy<manage_new_object>())
        .def("classof", classof_AllocationInst)
        .def("classof", classof_Instruction)
        .def("classof", classof_Value)
        .staticmethod("classof")
        ;
}

template MallocInst* InstInitFn4(const Type*, Value*, const std::string&, Instruction*);
template MallocInst* InstInitFn4(const Type*, Value*, const std::string&, BasicBlock*);

void wrap_MallocInst()
{
    // Name C++ overloads
    bool (*classof_MallocInst)(const MallocInst*) = &MallocInst::classof;
    bool (*classof_Instruction)(const Instruction*) = &MallocInst::classof;
    bool (*classof_Value)(const Value*) = &MallocInst::classof;

    class_<MallocInst, boost::noncopyable, bases<AllocationInst> >
        ("MallocInst", init<const Type*, optional<Value*, const std::string> >())
        .def("init", INST_INIT_DEF4(MallocInst, const Type*, Value*, const std::string&, Instruction*))
        .def("init", INST_INIT_DEF4(MallocInst, const Type*, Value*, const std::string&, BasicBlock*))
        .staticmethod("init")

        .def("clone", &MallocInst::clone,
             return_value_policy<manage_new_object>())
        .def("classof", classof_MallocInst)
        .def("classof", classof_Instruction)
        .def("classof", classof_Value)
        .staticmethod("classof")
        ;
}

template AllocaInst* InstInitFn4(const Type*, Value*, const std::string&, Instruction*);
template AllocaInst* InstInitFn4(const Type*, Value*, const std::string&, BasicBlock*);

void wrap_AllocaInst()
{
    // Name C++ overloads
    bool (*classof_AllocaInst)(const AllocaInst*) = &AllocaInst::classof;
    bool (*classof_Instruction)(const Instruction*) = &AllocaInst::classof;
    bool (*classof_Value)(const Value*) = &AllocaInst::classof;

    class_<AllocaInst, boost::noncopyable, bases<AllocationInst> >
        ("AllocaInst", init<const Type*, optional<Value*, const std::string> >())
        .def("init", INST_INIT_DEF4(AllocaInst, const Type*, Value*, const std::string&, Instruction*))
        .def("init", INST_INIT_DEF4(AllocaInst, const Type*, Value*, const std::string&, BasicBlock*))
        .staticmethod("init")

        .def("clone", &AllocaInst::clone,
             return_value_policy<manage_new_object>())
        .def("classof", classof_AllocaInst)
        .def("classof", classof_Instruction)
        .def("classof", classof_Value)
        .staticmethod("classof")
        ;
}

template FreeInst* InstInitFn2(Value*, Instruction*);
template FreeInst* InstInitFn2(Value*, BasicBlock*);

void wrap_FreeInst()
{
    // Name C++ overloads
    bool (*classof_FreeInst)(const FreeInst*) = &FreeInst::classof;
    bool (*classof_Instruction)(const Instruction*) = &FreeInst::classof;
    bool (*classof_Value)(const Value*) = &FreeInst::classof;

    class_<FreeInst, boost::noncopyable, bases<Instruction> >
        ("FreeInst", init<Value*>())
        .def("init", INST_INIT_DEF2(FreeInst, Value*, Instruction*))
        .def("init", INST_INIT_DEF2(FreeInst, Value*, BasicBlock*))
        .staticmethod("init")

        .def("clone", &FreeInst::clone,
             return_value_policy<manage_new_object>())
        .def("classof", classof_FreeInst)
        .def("classof", classof_Instruction)
        .def("classof", classof_Value)
        .staticmethod("classof")
        ;
}

template LoadInst* InstInitFn3(Value*, const std::string&, Instruction*);
template LoadInst* InstInitFn3(Value*, const std::string&, BasicBlock*);
template LoadInst* InstInitFn4(Value*, const std::string&, bool, Instruction*);
template LoadInst* InstInitFn4(Value*, const std::string&, bool, BasicBlock*);

void wrap_LoadInst()
{
    // Name C++ overloads
    const Value* (LoadInst::*getPointerOperand)() const = &LoadInst::getPointerOperand;
    bool (*classof_LoadInst)(const LoadInst*) = &LoadInst::classof;
    bool (*classof_Instruction)(const Instruction*) = &LoadInst::classof;
    bool (*classof_Value)(const Value*) = &LoadInst::classof;

    class_<LoadInst, boost::noncopyable, bases<Instruction> >
        ("LoadInst", init<Value*, optional<const std::string, bool> >())
        .def("init", INST_INIT_DEF3(LoadInst, Value*, const std::string&, Instruction*))
        .def("init", INST_INIT_DEF3(LoadInst, Value*, const std::string&, BasicBlock*))
        .def("init", INST_INIT_DEF4(LoadInst, Value*, const std::string&, bool, Instruction*))
        .def("init", INST_INIT_DEF4(LoadInst, Value*, const std::string&, bool, BasicBlock*))
        .staticmethod("init")

        .def("isVolatile", &LoadInst::isVolatile)
        .def("setVolatile", &LoadInst::setVolatile)
        .def("clone", &LoadInst::clone,
             return_value_policy<manage_new_object>())
        .def("mayWriteToMemory", &LoadInst::mayWriteToMemory)
        .def("getPointerOperand", getPointerOperand,
             return_internal_reference<>())
        .def("getPointerOperandIndex", &LoadInst::getPointerOperandIndex)
        .staticmethod("getPointerOperandIndex")
        .def("classof", classof_LoadInst)
        .def("classof", classof_Instruction)
        .def("classof", classof_Value)
        .staticmethod("classof")
        ;
}

template StoreInst* InstInitFn3(Value*, Value*, Instruction*);
template StoreInst* InstInitFn3(Value*, Value*, BasicBlock*);
template StoreInst* InstInitFn4(Value*, Value*, bool, Instruction*);
template StoreInst* InstInitFn4(Value*, Value*, bool, BasicBlock*);

void wrap_StoreInst()
{
    // Name C++ overloads
    const Value* (StoreInst::*getPointerOperand)() const = &StoreInst::getPointerOperand;
    bool (*classof_StoreInst)(const StoreInst*) = &StoreInst::classof;
    bool (*classof_Instruction)(const Instruction*) = &StoreInst::classof;
    bool (*classof_Value)(const Value*) = &StoreInst::classof;

    class_<StoreInst, boost::noncopyable, bases<Instruction> >
        ("StoreInst", init<Value*, Value*, optional<bool> >())
        .def("init", INST_INIT_DEF3(StoreInst, Value*, Value*, Instruction*))
        .def("init", INST_INIT_DEF3(StoreInst, Value*, Value*, BasicBlock*))
        .def("init", INST_INIT_DEF4(StoreInst, Value*, Value*, bool, Instruction*))
        .def("init", INST_INIT_DEF4(StoreInst, Value*, Value*, bool, BasicBlock*))
        .staticmethod("init")

        .def("isVolatile", &StoreInst::isVolatile)
        .def("setVolatile", &StoreInst::setVolatile)
        .def("clone", &StoreInst::clone,
             return_value_policy<manage_new_object>())
        .def("mayWriteToMemory", &StoreInst::mayWriteToMemory)
        .def("getPointerOperand", getPointerOperand,
             return_internal_reference<>())
        .def("getPointerOperandIndex", &StoreInst::getPointerOperandIndex)
        .staticmethod("getPointerOperandIndex")
        .def("classof", classof_StoreInst)
        .def("classof", classof_Instruction)
        .def("classof", classof_Value)
        .staticmethod("classof")
        ;
}

template GetElementPtrInst* InstInitFn4(Value*, const std::vector<Value*>&, const std::string&, Instruction*);
template GetElementPtrInst* InstInitFn4(Value*, const std::vector<Value*>&, const std::string&, BasicBlock*);
template GetElementPtrInst* InstInitFn5(Value*, Value*, Value*, const std::string&, Instruction*);
template GetElementPtrInst* InstInitFn5(Value*, Value*, Value*, const std::string&, BasicBlock*);

void wrap_GetElementPtrInst()
{
    // Name C++ overloads
    const Type* (*getIndexedType)(const Type*, const std::vector<Value*>&, bool) = &GetElementPtrInst::getIndexedType;
    const Type* (*getIndexedType2)(const Type*, Value*, Value*, bool) = &GetElementPtrInst::getIndexedType;
    GetElementPtrInst::const_op_iterator (GetElementPtrInst::*idx_begin)() const = &GetElementPtrInst::idx_begin;
    GetElementPtrInst::const_op_iterator (GetElementPtrInst::*idx_end)() const = &GetElementPtrInst::idx_end;
    const Value* (GetElementPtrInst::*getPointerOperand)() const = &GetElementPtrInst::getPointerOperand;
    bool (*classof_GetElementPtrInst)(const GetElementPtrInst*) = &GetElementPtrInst::classof;
    bool (*classof_Instruction)(const Instruction*) = &GetElementPtrInst::classof;
    bool (*classof_Value)(const Value*) = &GetElementPtrInst::classof;

    class_<GetElementPtrInst, boost::noncopyable, bases<Instruction> >
        ("GetElementPtrInst", init<Value*, const std::vector<Value*>&,
                                   optional<const std::string> >())
        .def(init<Value*, Value*, Value*,
                  optional<const std::string> >())
        .def("init", INST_INIT_DEF4(GetElementPtrInst, Value*, const std::vector<Value*>&, const std::string&, Instruction*))
        .def("init", INST_INIT_DEF4(GetElementPtrInst, Value*, const std::vector<Value*>&, const std::string&, BasicBlock*))
        .def("init", INST_INIT_DEF5(GetElementPtrInst, Value*, Value*, Value*, const std::string&, Instruction*))
        .def("init", INST_INIT_DEF5(GetElementPtrInst, Value*, Value*, Value*, const std::string&, BasicBlock*))
        .staticmethod("init")

        .def("clone", &GetElementPtrInst::clone,
             return_value_policy<manage_new_object>())
        .def("getType", &GetElementPtrInst::getType,
             return_internal_reference<>())
        .def("getIndexedType", getIndexedType,
             return_internal_reference<>())
        .def("getIndexedType", getIndexedType2,
             return_internal_reference<>())
        .staticmethod("getIndexedType")
        .add_property("indices", range(idx_begin, idx_end))
        //.def("idx_begin", idx_begin)
        //.def("idx_end", idx_end)
        .def("getPointerOperand", getPointerOperand,
             return_internal_reference<>())
        .def("getPointerOperandIndex", &GetElementPtrInst::getPointerOperandIndex)
        .staticmethod("getPointerOperandIndex")
        .def("getNumIndices", &GetElementPtrInst::getNumIndices)
        .def("hasIndices", &GetElementPtrInst::hasIndices)
        .def("classof", classof_GetElementPtrInst)
        .def("classof", classof_Instruction)
        .def("classof", classof_Value)
        .staticmethod("classof")
        ;
}

template SetCondInst* InstInitFn5(Instruction::BinaryOps, Value*, Value*, const std::string&, Instruction*);
template SetCondInst* InstInitFn5(Instruction::BinaryOps, Value*, Value*, const std::string&, BasicBlock*);

void wrap_SetCondInst()
{
    // Name C++ overloads
    Instruction::BinaryOps (SetCondInst::*getInverseCondition)() const = &SetCondInst::getInverseCondition;
    Instruction::BinaryOps (*getInverseCondition_static)(Instruction::BinaryOps) = &SetCondInst::getInverseCondition;
    Instruction::BinaryOps (SetCondInst::*getSwappedCondition)() const = &SetCondInst::getSwappedCondition;
    Instruction::BinaryOps (*getSwappedCondition_static)(Instruction::BinaryOps) = &SetCondInst::getSwappedCondition;
    bool (*classof_SetCondInst)(const SetCondInst*) = &SetCondInst::classof;
    bool (*classof_Instruction)(const Instruction*) = &SetCondInst::classof;
    bool (*classof_Value)(const Value*) = &SetCondInst::classof;

    class_<SetCondInst, boost::noncopyable, bases<BinaryOperator> >
        ("SetCondInst", init<Instruction::BinaryOps, Value*, Value*,
                             optional<const std::string> >())
        .def("init", INST_INIT_DEF5(SetCondInst, Instruction::BinaryOps, Value*, Value*, const std::string&, Instruction*))
        .def("init", INST_INIT_DEF5(SetCondInst, Instruction::BinaryOps, Value*, Value*, const std::string&, BasicBlock*))
        .staticmethod("init")

        .def("getInverseCondition", getInverseCondition)
        .def("getInverseCondition", getInverseCondition_static)
        .staticmethod("getInverseCondition")
        .def("getSwappedCondition", getSwappedCondition)
        .def("getSwappedCondition", getSwappedCondition_static)
        .staticmethod("getSwappedCondition")
        .def("classof", classof_SetCondInst)
        .def("classof", classof_Instruction)
        .def("classof", classof_Value)
        .staticmethod("classof")
        ;
}

template CastInst* InstInitFn4(Value*, const Type*, const std::string&, Instruction*);
template CastInst* InstInitFn4(Value*, const Type*, const std::string&, BasicBlock*);

void wrap_CastInst()
{
    // Name C++ overloads
    bool (*classof_CastInst)(const CastInst*) = &CastInst::classof;
    bool (*classof_Instruction)(const Instruction*) = &CastInst::classof;
    bool (*classof_Value)(const Value*) = &CastInst::classof;

    class_<CastInst, boost::noncopyable, bases<Instruction> >
        ("CastInst", init<Value*, const Type*,
                          optional<const std::string> >())
        .def("init", INST_INIT_DEF4(CastInst, Value*, const Type*, const std::string&, Instruction*))
        .def("init", INST_INIT_DEF4(CastInst, Value*, const Type*, const std::string&, BasicBlock*))
        .staticmethod("init")

        .def("clone", &CastInst::clone,
             return_value_policy<manage_new_object>())
        .def("classof", classof_CastInst)
        .def("classof", classof_Instruction)
        .def("classof", classof_Value)
        .staticmethod("classof")
        ;
}

template CallInst* InstInitFn4(Value*, const std::vector<Value*>&, const std::string&, Instruction*);
template CallInst* InstInitFn4(Value*, const std::vector<Value*>&, const std::string&, BasicBlock*);
template CallInst* InstInitFn5(Value*, Value*, Value*, const std::string&, Instruction*);
template CallInst* InstInitFn5(Value*, Value*, Value*, const std::string&, BasicBlock*);
template CallInst* InstInitFn4(Value*, Value*, const std::string&, Instruction*);
template CallInst* InstInitFn4(Value*, Value*, const std::string&, BasicBlock*);
template CallInst* InstInitFn3(Value*, const std::string&, Instruction*);
template CallInst* InstInitFn3(Value*, const std::string&, BasicBlock*);

void wrap_CallInst()
{
    // Name C++ overloads
    const Value* (CallInst::*getCalledValue)() const = &CallInst::getCalledValue;
    bool (*classof_CallInst)(const CallInst*) = &CallInst::classof;
    bool (*classof_Instruction)(const Instruction*) = &CallInst::classof;
    bool (*classof_Value)(const Value*) = &CallInst::classof;

    class_<CallInst, boost::noncopyable, bases<Instruction> >
        ("CallInst", init<Value*, const std::vector<Value*>&,
                          optional<const std::string> >())
        .def(init<Value*, Value*, Value*,
                  optional<const std::string> >())
        .def(init<Value*, Value*,
                  optional<const std::string> >())
        .def(init<Value*,
                  optional<const std::string> >())
        .def("init", INST_INIT_DEF4(CallInst, Value*, const std::vector<Value*>&, const std::string&, Instruction*))
        .def("init", INST_INIT_DEF4(CallInst, Value*, const std::vector<Value*>&, const std::string&, BasicBlock*))
        .def("init", INST_INIT_DEF5(CallInst, Value*, Value*, Value*, const std::string&, Instruction*))
        .def("init", INST_INIT_DEF5(CallInst, Value*, Value*, Value*, const std::string&, BasicBlock*))
        .def("init", INST_INIT_DEF4(CallInst, Value*, Value*, const std::string&, Instruction*))
        .def("init", INST_INIT_DEF4(CallInst, Value*, Value*, const std::string&, BasicBlock*))
        .def("init", INST_INIT_DEF3(CallInst, Value*, const std::string&, Instruction*))
        .def("init", INST_INIT_DEF3(CallInst, Value*, const std::string&, BasicBlock*))
        .staticmethod("init")

        .def("clone", &CallInst::clone,
             return_value_policy<manage_new_object>())
        .def("mayWriteToMemory", &CallInst::mayWriteToMemory)
        .def("getCalledFunction", &CallInst::getCalledFunction,
             return_internal_reference<>())
        .def("getCalledValue", getCalledValue,
             return_internal_reference<>())
        .def("classof", classof_CallInst)
        .def("classof", classof_Instruction)
        .def("classof", classof_Value)
        .staticmethod("classof")
        ;
}

template ShiftInst* InstInitFn5(Instruction::OtherOps, Value*, Value*, const std::string&, Instruction*);
template ShiftInst* InstInitFn5(Instruction::OtherOps, Value*, Value*, const std::string&, BasicBlock*);

void wrap_ShiftInst()
{
    // Name C++ overloads
    bool (*classof_ShiftInst)(const ShiftInst*) = &ShiftInst::classof;
    bool (*classof_Instruction)(const Instruction*) = &ShiftInst::classof;
    bool (*classof_Value)(const Value*) = &ShiftInst::classof;

    class_<ShiftInst, boost::noncopyable, bases<Instruction> >
        ("ShiftInst", init<Instruction::OtherOps, Value*, Value*,
                           optional<const std::string> >())
        .def("init", INST_INIT_DEF5(ShiftInst, Instruction::OtherOps, Value*, Value*, const std::string&, Instruction*))
        .def("init", INST_INIT_DEF5(ShiftInst, Instruction::OtherOps, Value*, Value*, const std::string&, BasicBlock*))
        .staticmethod("init")

        .def("getOpcode", &ShiftInst::getOpcode)
        .def("clone", &ShiftInst::clone,
             return_value_policy<manage_new_object>())
        .def("classof", classof_ShiftInst)
        .def("classof", classof_Instruction)
        .def("classof", classof_Value)
        .staticmethod("classof")
        ;
}

template SelectInst* InstInitFn5(Value*, Value*, Value*, const std::string&, Instruction*);
template SelectInst* InstInitFn5(Value*, Value*, Value*, const std::string&, BasicBlock*);

void wrap_SelectInst()
{
    // Name C++ overloads
    bool (*classof_SelectInst)(const SelectInst*) = &SelectInst::classof;
    bool (*classof_Instruction)(const Instruction*) = &SelectInst::classof;
    bool (*classof_Value)(const Value*) = &SelectInst::classof;

    class_<SelectInst, boost::noncopyable, bases<Instruction> >
        ("SelectInst", init<Value*, Value*, Value*,
                            optional<const std::string> >())
        .def("init", INST_INIT_DEF5(SelectInst, Value*, Value*, Value*, const std::string&, Instruction*))
        .def("init", INST_INIT_DEF5(SelectInst, Value*, Value*, Value*, const std::string&, BasicBlock*))
        .staticmethod("init")

        .def("getCondition", &SelectInst::getCondition,
             return_internal_reference<>())
        .def("getTrueValue", &SelectInst::getTrueValue,
             return_internal_reference<>())
        .def("getFalseValue", &SelectInst::getFalseValue,
             return_internal_reference<>())
        .def("getOpcode", &SelectInst::getOpcode)
        .def("clone", &SelectInst::clone,
             return_value_policy<manage_new_object>())
        .def("classof", classof_SelectInst)
        .def("classof", classof_Instruction)
        .def("classof", classof_Value)
        .staticmethod("classof")
        ;
}

template VANextInst* InstInitFn4(Value*, const Type*, const std::string&, Instruction*);
template VANextInst* InstInitFn4(Value*, const Type*, const std::string&, BasicBlock*);

void wrap_VANextInst()
{
    // Name C++ overloads
    bool (*classof_VANextInst)(const VANextInst*) = &VANextInst::classof;
    bool (*classof_Instruction)(const Instruction*) = &VANextInst::classof;
    bool (*classof_Value)(const Value*) = &VANextInst::classof;

    class_<VANextInst, boost::noncopyable, bases<Instruction> >
        ("VANextInst", init<Value*, const Type*,
                            optional<const std::string> >())
        .def("init", INST_INIT_DEF4(VANextInst, Value*, const Type*, const std::string&, Instruction*))
        .def("init", INST_INIT_DEF4(VANextInst, Value*, const Type*, const std::string&, BasicBlock*))
        .staticmethod("init")

        .def("getArgType", &VANextInst::getArgType,
             return_internal_reference<>())
        .def("clone", &VANextInst::clone,
             return_value_policy<manage_new_object>())
        .def("classof", classof_VANextInst)
        .def("classof", classof_Instruction)
        .def("classof", classof_Value)
        .staticmethod("classof")
        ;
}

template VAArgInst* InstInitFn4(Value*, const Type*, const std::string&, Instruction*);
template VAArgInst* InstInitFn4(Value*, const Type*, const std::string&, BasicBlock*);

void wrap_VAArgInst()
{
    // Name C++ overloads
    bool (*classof_VAArgInst)(const VAArgInst*) = &VAArgInst::classof;
    bool (*classof_Instruction)(const Instruction*) = &VAArgInst::classof;
    bool (*classof_Value)(const Value*) = &VAArgInst::classof;

    class_<VAArgInst, boost::noncopyable, bases<Instruction> >
        ("VAArgInst", init<Value*, const Type*,
                            optional<const std::string> >())
        .def("init", INST_INIT_DEF4(VAArgInst, Value*, const Type*, const std::string&, Instruction*))
        .def("init", INST_INIT_DEF4(VAArgInst, Value*, const Type*, const std::string&, BasicBlock*))
        .staticmethod("init")

        .def("clone", &VAArgInst::clone,
             return_value_policy<manage_new_object>())
        .def("classof", classof_VAArgInst)
        .def("classof", classof_Instruction)
        .def("classof", classof_Value)
        .staticmethod("classof")
        ;
}

template PHINode* InstInitFn3(const Type*, const std::string&, Instruction*);
template PHINode* InstInitFn3(const Type*, const std::string&, BasicBlock*);

void wrap_PHINode()
{
    // Name C++ overloads
    Value* (PHINode::*removeIncomingValue)(unsigned, bool) = &PHINode::removeIncomingValue;
    Value* (PHINode::*removeIncomingValue2)(const BasicBlock*, bool) = &PHINode::removeIncomingValue;
    bool (*classof_PHINode)(const PHINode*) = &PHINode::classof;
    bool (*classof_Instruction)(const Instruction*) = &PHINode::classof;
    bool (*classof_Value)(const Value*) = &PHINode::classof;

    class_<PHINode, boost::noncopyable, bases<Instruction> >
        ("PHINode", init<const Type*,
                         optional<const std::string> >())
        .def("init", INST_INIT_DEF3(PHINode, const Type*, const std::string&, Instruction*))
        .def("init", INST_INIT_DEF3(PHINode, const Type*, const std::string&, BasicBlock*))
        .staticmethod("init")

        .def("clone", &PHINode::clone,
             return_value_policy<manage_new_object>())
        .def("getNumIncomingValues", &PHINode::getNumIncomingValues)
        .def("getIncomingValue", &PHINode::getIncomingValue,
             return_internal_reference<>())
        .def("setIncomingValue", &PHINode::setIncomingValue)
        .def("getOperandNumForIncomingValue", &PHINode::getOperandNumForIncomingValue)
        .def("getIncomingBlock", &PHINode::getIncomingBlock,
             return_internal_reference<>())
        .def("setIncomingBlock", &PHINode::setIncomingBlock)
        .def("getOperandNumForIncomingBlock", &PHINode::getOperandNumForIncomingBlock)
        .def("addIncoming", &PHINode::addIncoming)
        .def("removeIncomingValue", removeIncomingValue,
             return_value_policy<manage_new_object>())
        .def("removeIncomingValue", removeIncomingValue2,
             return_value_policy<manage_new_object>())
        .def("getBasicBlockIndex", &PHINode::getBasicBlockIndex)
        .def("getIncomingValueForBlock", &PHINode::getIncomingValueForBlock,
             return_internal_reference<>())
        .def("classof", classof_PHINode)
        .def("classof", classof_Instruction)
        .def("classof", classof_Value)
        .staticmethod("classof")
        ;
}

template ReturnInst* InstInitFn2(Value *RetVal, Instruction *InsertBefore);
template ReturnInst* InstInitFn2(Value *RetVal, BasicBlock *InsertAtEnd);
template ReturnInst* InstInitFn1(BasicBlock *InsertAtEnd);

void wrap_ReturnInst()
{
    // Name C++ overloads
    bool (*classof_ReturnInst)(const ReturnInst*) = &ReturnInst::classof;
    bool (*classof_Instruction)(const Instruction*) = &ReturnInst::classof;
    bool (*classof_Value)(const Value*) = &ReturnInst::classof;
    
    class_<ReturnInst, std::auto_ptr<ReturnInst>, boost::noncopyable, bases<TerminatorInst> >
        ("ReturnInst", init<optional<Value*> >())
        .def("init", INST_INIT_DEF2(ReturnInst, Value*, Instruction*))
        .def("init", INST_INIT_DEF2(ReturnInst, Value*, BasicBlock*))
        .def("init", INST_INIT_DEF1(ReturnInst, BasicBlock*))
        .staticmethod("init")

        .def("clone", &ReturnInst::clone,
             return_value_policy<manage_new_object>())
        .def("getReturnValue", &ReturnInst::getReturnValue,
             return_internal_reference<>())
        .def("getSuccessor", &ReturnInst::getSuccessor,
             return_internal_reference<>())
        .def("setSuccessor", &ReturnInst::setSuccessor)
        .def("getNumSuccessors", &ReturnInst::getNumSuccessors)
        .def("classof", classof_ReturnInst)
        .def("classof", classof_Instruction)
        .def("classof", classof_Value)
        .staticmethod("classof")
        ;
}

template BranchInst* InstInitFn2(BasicBlock*, Instruction*);
template BranchInst* InstInitFn4(BasicBlock*, BasicBlock*, Value*, Instruction*);
template BranchInst* InstInitFn2(BasicBlock*, BasicBlock*);
template BranchInst* InstInitFn4(BasicBlock*, BasicBlock*, Value*, BasicBlock*);

void wrap_BranchInst()
{
    // Name C++ overloads
    bool (*classof_BranchInst)(const BranchInst*) = &BranchInst::classof;
    bool (*classof_Instruction)(const Instruction*) = &BranchInst::classof;
    bool (*classof_Value)(const Value*) = &BranchInst::classof;

    class_<BranchInst, boost::noncopyable, bases<TerminatorInst> >
        ("BranchInst", init<BasicBlock*>())
        .def(init<BasicBlock*, BasicBlock*, Value*>())
        .def("init", INST_INIT_DEF2(BranchInst, BasicBlock*, Instruction*))
        .def("init", INST_INIT_DEF4(BranchInst, BasicBlock*, BasicBlock*, Value*, Instruction*))
        .def("init", INST_INIT_DEF2(BranchInst, BasicBlock*, BasicBlock*))
        .def("init", INST_INIT_DEF4(BranchInst, BasicBlock*, BasicBlock*, Value*, BasicBlock*))
        .staticmethod("init")

        .def("clone", &BranchInst::clone,
             return_value_policy<manage_new_object>())
        .def("isUnconditional", &BranchInst::isUnconditional)
        .def("isConditional", &BranchInst::isConditional)
        .def("getCondition", &BranchInst::getCondition,
             return_internal_reference<>())
        .def("setCondition", &BranchInst::setCondition)
        .def("setUnconditionalDest", &BranchInst::setUnconditionalDest)
        .def("getSuccessor", &BranchInst::getSuccessor,
             return_internal_reference<>())
        .def("setSuccessor", &BranchInst::setSuccessor)
        .def("getNumSuccessors", &BranchInst::getNumSuccessors)
        .def("classof", classof_BranchInst)
        .def("classof", classof_Instruction)
        .def("classof", classof_Value)
        .staticmethod("classof")
        ;
}

template SwitchInst* InstInitFn4(Value*, BasicBlock*, unsigned, Instruction*);
template SwitchInst* InstInitFn4(Value*, BasicBlock*, unsigned, BasicBlock *);

void wrap_SwitchInst()
{
    // Name C++ overloads
    const Constant* (SwitchInst::*getCaseValue)(unsigned) const = &SwitchInst::getCaseValue;
    bool (*classof_SwitchInst)(const SwitchInst*) = &SwitchInst::classof;
    bool (*classof_Instruction)(const Instruction*) = &SwitchInst::classof;
    bool (*classof_Value)(const Value*) = &SwitchInst::classof;

    class_<SwitchInst, boost::noncopyable, bases<TerminatorInst> >
        ("SwitchInst", init<Value*, BasicBlock*, unsigned>())
        .def("init", INST_INIT_DEF4(SwitchInst, Value*, BasicBlock*, unsigned, Instruction*))
        .def("init", INST_INIT_DEF4(SwitchInst, Value*, BasicBlock*, unsigned, BasicBlock *))
        .staticmethod("init")

        .def("clone", &SwitchInst::clone,
             return_value_policy<manage_new_object>())
        .def("getCondition", &SwitchInst::getCondition,
             return_internal_reference<>())
        .def("getDefaultDest", &SwitchInst::getDefaultDest,
             return_internal_reference<>())
        .def("getNumCases", &SwitchInst::getNumCases)
        .def("getCaseValue", getCaseValue,
             return_internal_reference<>())
        .def("findCaseValue", &SwitchInst::findCaseValue)
        .def("addCase", &SwitchInst::addCase)
        .def("removeCase", &SwitchInst::removeCase)
        .def("getSuccessor", &SwitchInst::getSuccessor,
             return_internal_reference<>())
        .def("setSuccessor", &SwitchInst::setSuccessor)
        .def("getSuccessorValue", &SwitchInst::getSuccessorValue,
             return_internal_reference<>())
        .def("getNumSuccessors", &SwitchInst::getNumSuccessors)
        .def("classof", classof_SwitchInst)
        .def("classof", classof_Instruction)
        .def("classof", classof_Value)
        .staticmethod("classof")
        ;
}

template InvokeInst* InstInitFn6(Value*, BasicBlock*, BasicBlock*, const std::vector<Value*>&, const std::string&, Instruction*);
template InvokeInst* InstInitFn6(Value*, BasicBlock*, BasicBlock*, const std::vector<Value*>&, const std::string&, BasicBlock*);

void wrap_InvokeInst()
{
    // Name C++ overloads
    bool (*classof_InvokeInst)(const InvokeInst*) = &InvokeInst::classof;
    bool (*classof_Instruction)(const Instruction*) = &InvokeInst::classof;
    bool (*classof_Value)(const Value*) = &InvokeInst::classof;

    class_<InvokeInst, boost::noncopyable, bases<TerminatorInst> >
        ("InvokeInst", init<Value*, BasicBlock*, BasicBlock*, const std::vector<Value*>&,
                            optional<const std::string> >())
        .def("init", INST_INIT_DEF6(InvokeInst, Value*, BasicBlock*, BasicBlock*, const std::vector<Value*>&, const std::string&, Instruction*))
        .def("init", INST_INIT_DEF6(InvokeInst, Value*, BasicBlock*, BasicBlock*, const std::vector<Value*>&, const std::string&, BasicBlock*))
        .staticmethod("init")

        .def("clone", &InvokeInst::clone,
             return_value_policy<manage_new_object>())
        .def("mayWriteToMemory", &InvokeInst::mayWriteToMemory)
        .def("getCalledFunction", &InvokeInst::getCalledFunction,
             return_internal_reference<>())
        .def("getCalledValue", &InvokeInst::getCalledValue,
             return_internal_reference<>())
        .def("getNormalDest", &InvokeInst::getNormalDest,
             return_internal_reference<>())
        .def("getUnwindDest", &InvokeInst::getUnwindDest,
             return_internal_reference<>())
        .def("setNormalDest", &InvokeInst::setNormalDest)
        .def("setUnwindDest", &InvokeInst::setUnwindDest)
        .def("getSuccessor", &InvokeInst::getSuccessor,
             return_internal_reference<>())
        .def("setSuccessor", &InvokeInst::setSuccessor)
        .def("getNumSuccessors", &InvokeInst::getNumSuccessors)
        .def("classof", classof_InvokeInst)
        .def("classof", classof_Instruction)
        .def("classof", classof_Value)
        .staticmethod("classof")
        ;
}

template UnwindInst* InstInitFn1(Instruction*);
template UnwindInst* InstInitFn1(BasicBlock*);

void wrap_UnwindInst()
{
    // Name C++ overloads
    bool (*classof_UnwindInst)(const UnwindInst*) = &UnwindInst::classof;
    bool (*classof_Instruction)(const Instruction*) = &UnwindInst::classof;
    bool (*classof_Value)(const Value*) = &UnwindInst::classof;

    class_<UnwindInst, boost::noncopyable, bases<TerminatorInst> >
        ("UnwindInst")
        .def("init", INST_INIT_DEF1(UnwindInst, Instruction*))
        .def("init", INST_INIT_DEF1(UnwindInst, BasicBlock*))
        .staticmethod("init")

        .def("clone", &UnwindInst::clone,
             return_value_policy<manage_new_object>())
        .def("getSuccessor", &UnwindInst::getSuccessor,
             return_internal_reference<>())
        .def("setSuccessor", &UnwindInst::setSuccessor)
        .def("getNumSuccessors", &UnwindInst::getNumSuccessors)
        .def("classof", classof_UnwindInst)
        .def("classof", classof_Instruction)
        .def("classof", classof_Value)
        .staticmethod("classof")
        ;
}

template UnreachableInst* InstInitFn1(Instruction*);
template UnreachableInst* InstInitFn1(BasicBlock*);

void wrap_UnreachableInst()
{
    // Name C++ overloads
    bool (*classof_UnreachableInst)(const UnreachableInst*) = &UnreachableInst::classof;
    bool (*classof_Instruction)(const Instruction*) = &UnreachableInst::classof;
    bool (*classof_Value)(const Value*) = &UnreachableInst::classof;

    class_<UnreachableInst, boost::noncopyable, bases<TerminatorInst> >
        ("UnreachableInst")
        .def("init", INST_INIT_DEF1(UnreachableInst, Instruction*))
        .def("init", INST_INIT_DEF1(UnreachableInst, BasicBlock*))
        .staticmethod("init")

        .def("clone", &UnreachableInst::clone,
             return_value_policy<manage_new_object>())
        .def("getSuccessor", &UnreachableInst::getSuccessor,
             return_internal_reference<>())
        .def("setSuccessor", &UnreachableInst::setSuccessor)
        .def("getNumSuccessors", &UnreachableInst::getNumSuccessors)
        .def("classof", classof_UnreachableInst)
        .def("classof", classof_Instruction)
        .def("classof", classof_Value)
        .staticmethod("classof")
        ;
}

void wrap_Instructions()
{
    wrap_AllocationInst();
    wrap_MallocInst();
    wrap_AllocaInst();
    wrap_FreeInst();
    wrap_LoadInst();
    wrap_StoreInst();
    wrap_GetElementPtrInst();
    wrap_SetCondInst();
    wrap_CastInst();
    wrap_CallInst();
    wrap_ShiftInst();
    wrap_SelectInst();
    wrap_VANextInst();
    wrap_VAArgInst();
    wrap_PHINode();
    wrap_ReturnInst();
    wrap_BranchInst();
    wrap_SwitchInst();
    wrap_InvokeInst();
    wrap_UnwindInst();
    wrap_UnreachableInst();
}

}} // namespace llvm::Python

// EOF
