/*
 * Linker.cpp: Python bindings for the LLVM compiler infrastructure
 * Copyright (c) 2004, 2005 Jarno Seppänen <firstname.lastname@iki.fi>.
 * Distributed under the Boost Software License, Version 1.0. (See
 * accompanying file LICENSE_1_0.txt or copy at
 * http://www.boost.org/LICENSE_1_0.txt)
 */

#include <boost/python.hpp>
#include <boost/noncopyable.hpp>
#include <llvm/Linker.h>
#include <llvm/Module.h>

#include "scitbx/container_conversions.h"

using namespace boost::python;
using namespace scitbx::boost_python::container_conversions;

namespace llvm { namespace Python {

/** Src ownership transfer. */
bool Linker_LinkInModule(Linker& l, std::auto_ptr<Module> Src)
{
    bool result = l.LinkInModule(Src.get());
    Src.release();
    return result;
}

/** Src ownership transfer. */
bool Linker_LinkModules(Linker& l, Module* Dest, std::auto_ptr<Module> Src, std::string* ErrorMsg)
{
    bool result = l.LinkModules(Dest, Src.get(), ErrorMsg);
    Src.release();
    return result;
}

void wrap_Linker()
{
    scope LinkerScope
        = class_<Linker, boost::noncopyable>
        ("Linker",
         init<std::string, optional<unsigned> >())

        .def(init<const std::string&, Module*, optional<unsigned> >())
        .def("getModule", &Linker::getModule,
             return_internal_reference<>())
        .def("releaseModule", &Linker::releaseModule,
             return_value_policy<manage_new_object>())
        .def("getLibPaths", &Linker::getLibPaths,
             return_value_policy<copy_const_reference>())
        .def("getLastError", &Linker::getLastError,
             return_value_policy<return_by_value>())
        .def("addPath", &Linker::addPath)
        .def("addPaths", &Linker::addPaths)
        .def("addSystemPaths", &Linker::addSystemPaths)
        .def("setFlags", &Linker::setFlags)
        .def("LinkInItems", &Linker::LinkInItems)
        .def("LinkInFiles", &Linker::LinkInFiles)
        .def("LinkInFile", &Linker::LinkInFile)
        .def("LinkInLibraries", &Linker::LinkInLibraries)
        .def("LinkInLibrary", &Linker::LinkInLibrary)
        .def("LinkInArchive", &Linker::LinkInArchive)
        .def("LinkInModule", Linker_LinkInModule) // Src ownership transfer
        .def("LinkModules", Linker_LinkModules) // Src ownership transfer
        .staticmethod("LinkModules")
        .def("FindLib", &Linker::FindLib)
        ;

    enum_<Linker::ControlFlags>("ControlFlags")
        .value("Verbose", Linker::Verbose)
        .value("QuietWarnings", Linker::QuietWarnings)
        .value("QuietErrors", Linker::QuietErrors)
        ;

    // Register tuple conversion for vector<sys::Path> return values
    to_python_converter<std::vector<sys::Path>,
                        to_tuple<std::vector<sys::Path> > >();

    // Register tuple conversion for vector<std::string> arguments
    from_python_sequence<std::vector<std::string>,
                         variable_capacity_policy>();

    // Register tuple conversion for vector<sys::Path> arguments
    from_python_sequence<std::vector<sys::Path>,
                         variable_capacity_policy>();
}
}} // namespace llvm::Python

// EOF
