/*
 * Pass.cpp: Python bindings for the LLVM compiler infrastructure
 * Copyright (c) 2004, 2005 Jarno Seppänen <firstname.lastname@iki.fi>.
 * Distributed under the Boost Software License, Version 1.0. (See
 * accompanying file LICENSE_1_0.txt or copy at
 * http://www.boost.org/LICENSE_1_0.txt)
 */

#include <boost/python.hpp>
#include <boost/python/make_constructor.hpp>
#include <boost/noncopyable.hpp>
#include <llvm/Pass.h>
//#include "../VMCore/PassManagerT.h"
#include <llvm/Module.h>
#include <llvm/BasicBlock.h>

using namespace boost::python;

namespace llvm { namespace Python {

// Allow virtual functions within Python
struct PassWrap : public Pass, wrapper<Pass>
{
    virtual bool runPass(Module& m) {
        // WARNING: Python callback mustn't store m beyond this method
        // (see http://www.boost.org/libs/python/doc/v2/callbacks.html)
        if (override o = this->get_override("runPass")) {
            return o(boost::ref(m));
        }
        return Pass::runPass(m);
    }
    virtual bool runPass(BasicBlock& bb) {
        // WARNING: Python callback mustn't store bb beyond this method
        // (see http://www.boost.org/libs/python/doc/v2/callbacks.html)
        if (override o = this->get_override("runPass")) {
            return o(boost::ref(bb));
        }
        return Pass::runPass(bb);
    }
    virtual void getAnalysisUsage(AnalysisUsage& au) const {
        // WARNING: Python callback mustn't store au beyond this method
        // (see http://www.boost.org/libs/python/doc/v2/callbacks.html)
        if (override o = this->get_override("getAnalysisUsage")) {
            o(boost::ref(au));
        }
        else {
            Pass::getAnalysisUsage(au);
        }
    }
    virtual void releaseMemory() {
        if (override o = this->get_override("releaseMemory")) {
            o();
        }
        else {
            Pass::releaseMemory();
        }
    }
    virtual void dumpPassStructure(unsigned Offset) {
        if (override o = this->get_override("dumpPassStructure")) {
            o(Offset);
        }
        else {
            Pass::dumpPassStructure(Offset);
        }
    }
    bool default_runPass1(Module& m) {
        return this->Pass::runPass(m);
    }
    bool default_runPass2(BasicBlock& bb) {
        return this->Pass::runPass(bb);
    }
    void default_getAnalysisUsage(AnalysisUsage& au) {
        this->Pass::getAnalysisUsage(au);
    }
    void default_releaseMemory() {
        this->Pass::releaseMemory();
    }
    void default_dumpPassStructure1() {
        this->Pass::dumpPassStructure();
    }
    void default_dumpPassStructure2(unsigned Offset) {
        this->Pass::dumpPassStructure(Offset);
    }

    /*
    void dumpPassStructure1() {
        dumpPassStructure();
    }
    void dumpPassStructure2(unsigned Offset) {
        dumpPassStructure(Offset);
    }
    */
};

void wrap_PassBase()
{
    // Name C++ overloads
    bool (Pass::*runPass1)(Module&) = &Pass::runPass;
    bool (Pass::*runPass2)(BasicBlock&) = &Pass::runPass;

    class_<PassWrap, std::auto_ptr<PassWrap>, boost::noncopyable>
        ("Pass")

        .def("getPassName", &Pass::getPassName)
        .def("getPassInfo", &Pass::getPassInfo,
             return_internal_reference<>())
        .def("runPass", runPass1, &PassWrap::default_runPass1)
        .def("runPass", runPass2, &PassWrap::default_runPass2)
        .def("print_", &Pass::print)
        .def("dump", &Pass::dump)
        //FIXME
        //.def("getAnalysisUsage", &Pass::getAnalysisUsage, &PassWrap::default_getAnalysisUsage)
        .def("releaseMemory", &Pass::releaseMemory, &PassWrap::default_releaseMemory)
        //FIXME
        //.def("dumpPassStructure", &Pass::dumpPassStructure1, &PassWrap::default_dumpPassStructure1)
        //.def("dumpPassStructure", &Pass::dumpPassStructure2, &PassWrap::default_dumpPassStructure2)
        //FIXME cannot wrap template method (use PyObject?)
        //.def("getClassPassInfo", &Pass::getClassPassInfo,
        //     return_internal_reference<>())
        //.staticmethod("getClassPassInfo")
        .def("lookupPassInfo", &Pass::lookupPassInfo,
             return_internal_reference<>())
        .staticmethod("lookupPassInfo")
        //.def("getAnalysisToUpdate", &Pass::getAnalysisToUpdate,
        //     return_internal_reference<>())
        .def("mustPreserveAnalysisID", &Pass::mustPreserveAnalysisID)
        //FIXME cannot wrap template method (use PyObject?)
        //.def("getAnalysis", &Pass::getAnalysis,
        //     return_internal_reference<>())
        //FIXME cannot wrap template method (use PyObject?)
        //.def("getAnalysisID", &Pass::getAnalysisID,
        //     return_internal_reference<>())
        ;

    // define upcasts
    implicitly_convertible<std::auto_ptr<PassWrap>, std::auto_ptr<Pass> >();
}

// Allow virtual functions within Python
struct ModulePassWrap : public ModulePass, wrapper<ModulePass>
{
    virtual bool runOnModule(Module& m) {
        // WARNING: Python callback mustn't store m beyond this method
        // (see http://www.boost.org/libs/python/doc/v2/callbacks.html)
        return this->get_override("runOnModule")(boost::ref(m));
    }
    operator const PassWrap&() const {
        return static_cast<const PassWrap&>(dynamic_cast<const Pass&>(*this));
    }
};

void wrap_ModulePass()
{
    // Name C++ overloads
    bool (ModulePass::*runPass1)(Module&) = &ModulePass::runPass;
    bool (ModulePass::*runPass2)(BasicBlock&) = &ModulePass::runPass;

    class_<ModulePassWrap, std::auto_ptr<ModulePassWrap>, boost::noncopyable, bases<Pass> >
        ("ModulePass")

        .def("runOnModule", pure_virtual(&ModulePass::runOnModule))
        .def("runPass", runPass1)
        .def("runPass", runPass2)
        // FIXME error: invalid use of undefined type `struct llvm::PassManagerT<llvm::Module>'
        //.def("addToPassManager", &ModulePass::addToPassManager)
        ;

    // define upcasts
    //implicitly_convertible<std::auto_ptr<ModulePassWrap>, std::auto_ptr<PassWrap> >();
    implicitly_convertible<std::auto_ptr<ModulePassWrap>, std::auto_ptr<ModulePass> >();
    implicitly_convertible<std::auto_ptr<ModulePass>, std::auto_ptr<Pass> >();
}

// Allow virtual functions within Python
struct ImmutablePassWrap : public ImmutablePass, wrapper<ImmutablePass>
{
    virtual void initializePass() {
        this->get_override("initializePass")();
    }
    operator const ModulePassWrap&() const {
        return static_cast<const ModulePassWrap&>(dynamic_cast<const ModulePass&>(*this));
    }
};

void wrap_ImmutablePass()
{
    class_<ImmutablePassWrap, std::auto_ptr<ImmutablePassWrap>, boost::noncopyable, bases<ModulePass> >
        ("ImmutablePass")

        .def("initializePass", pure_virtual(&ImmutablePass::initializePass))
        .def("runOnModule", &ImmutablePass::runOnModule)
        ;

    // define upcasts
    //implicitly_convertible<std::auto_ptr<ImmutablePassWrap>, std::auto_ptr<ModulePassWrap> >();
    implicitly_convertible<std::auto_ptr<ImmutablePassWrap>, std::auto_ptr<ImmutablePass> >();
    implicitly_convertible<std::auto_ptr<ImmutablePass>, std::auto_ptr<ModulePass> >();
}

// Allow virtual functions within Python
struct FunctionPassWrap : public FunctionPass, wrapper<FunctionPass>
{
    virtual bool doInitialization(Module& m) {
        // WARNING: Python callback mustn't store m beyond this method
        // (see http://www.boost.org/libs/python/doc/v2/callbacks.html)
        if (override o = this->get_override("doInitialization")) {
            return o(boost::ref(m));
        }
        return FunctionPass::doInitialization(m);
    }
    virtual bool runOnFunction(Function& f) {
        // WARNING: Python callback mustn't store f beyond this method
        // (see http://www.boost.org/libs/python/doc/v2/callbacks.html)
        return this->get_override("runOnFunction")(boost::ref(f));
    }
    virtual bool doFinalization(Module& m) {
        // WARNING: Python callback mustn't store m beyond this method
        // (see http://www.boost.org/libs/python/doc/v2/callbacks.html)
        if (override o = this->get_override("doFinalization")) {
            return o(boost::ref(m));
        }
        return FunctionPass::doFinalization(m);
    }
    operator const ModulePassWrap&() const {
        return static_cast<const ModulePassWrap&>(dynamic_cast<const ModulePass&>(*this));
    }

    bool default_doInitialization(Module& m) {
        return this->FunctionPass::doInitialization(m);
    }
    bool default_doFinalization(Module& m) {
        return this->FunctionPass::doFinalization(m);
    }
};

void wrap_FunctionPass()
{
    class_<FunctionPassWrap, std::auto_ptr<FunctionPassWrap>, boost::noncopyable, bases<ModulePass> >
        ("FunctionPass")

        .def("doInitialization",
             &FunctionPass::doInitialization, &FunctionPassWrap::default_doInitialization)
        .def("runOnFunction", pure_virtual(&FunctionPass::runOnFunction))
        .def("doFinalization",
             &FunctionPass::doFinalization, &FunctionPassWrap::default_doFinalization)
        .def("runOnModule", &FunctionPass::runOnModule)
        .def("run", &FunctionPass::run)
        ;

    // define upcasts
    //implicitly_convertible<std::auto_ptr<FunctionPassWrap>, std::auto_ptr<ModulePassWrap> >();
    implicitly_convertible<std::auto_ptr<FunctionPassWrap>, std::auto_ptr<FunctionPass> >();
    implicitly_convertible<std::auto_ptr<FunctionPass>, std::auto_ptr<ModulePass> >();
}

// Allow virtual functions within Python
struct BasicBlockPassWrap : public BasicBlockPass, wrapper<BasicBlockPass>
{
    virtual bool doInitialization(Module& m) {
        // WARNING: Python callback mustn't store m beyond this method
        // (see http://www.boost.org/libs/python/doc/v2/callbacks.html)
        return this->get_override("doInitialization")(boost::ref(m));
    }
    virtual bool doInitialization(Function& f) {
        // WARNING: Python callback mustn't store f beyond this method
        // (see http://www.boost.org/libs/python/doc/v2/callbacks.html)
        return this->get_override("doInitialization")(boost::ref(f));
    }
    virtual bool runOnBasicBlock(BasicBlock& bb) {
        // WARNING: Python callback mustn't store bb beyond this method
        // (see http://www.boost.org/libs/python/doc/v2/callbacks.html)
        return this->get_override("runOnBasicBlock")(boost::ref(bb));
    }
    virtual bool doFinalization(Function& f) {
        // WARNING: Python callback mustn't store f beyond this method
        // (see http://www.boost.org/libs/python/doc/v2/callbacks.html)
        return this->get_override("doFinalization")(boost::ref(f));
    }
    virtual bool doFinalization(Module& m) {
        // WARNING: Python callback mustn't store m beyond this method
        // (see http://www.boost.org/libs/python/doc/v2/callbacks.html)
        return this->get_override("doFinalization")(boost::ref(m));
    }
    operator const FunctionPassWrap&() const {
        return static_cast<const FunctionPassWrap&>(dynamic_cast<const FunctionPass&>(*this));
    }
};

void wrap_BasicBlockPass()
{
    // Name C++ overloads
    bool (BasicBlockPass::*doInitialization1)(Module&) = &BasicBlockPass::doInitialization;
    bool (BasicBlockPass::*doInitialization2)(Function&) = &BasicBlockPass::doInitialization;
    bool (BasicBlockPass::*doFinalization1)(Module&) = &BasicBlockPass::doFinalization;
    bool (BasicBlockPass::*doFinalization2)(Function&) = &BasicBlockPass::doFinalization;
    bool (BasicBlockPass::*runPass1)(Module&) = &BasicBlockPass::runPass;
    bool (BasicBlockPass::*runPass2)(BasicBlock&) = &BasicBlockPass::runPass;

    class_<BasicBlockPassWrap, std::auto_ptr<BasicBlockPassWrap>, boost::noncopyable, bases<FunctionPass> >
        ("BasicBlockPass")

        .def("doInitialization", pure_virtual(doInitialization1))
        .def("doInitialization", pure_virtual(doInitialization2))
        .def("runOnBasicBlock", pure_virtual(&BasicBlockPass::runOnBasicBlock))
        .def("doFinalization", pure_virtual(doFinalization1))
        .def("doFinalization", pure_virtual(doFinalization2))
        .def("runOnFunction", &BasicBlockPass::runOnFunction)
        .def("runPass", runPass1)
        .def("runPass", runPass2)
        ;

    // define upcasts
    //implicitly_convertible<std::auto_ptr<BasicBlockPassWrap>, std::auto_ptr<FunctionPassWrap> >();
    implicitly_convertible<std::auto_ptr<BasicBlockPassWrap>, std::auto_ptr<BasicBlockPass> >();
    implicitly_convertible<std::auto_ptr<BasicBlockPass>, std::auto_ptr<FunctionPass> >();
}

void wrap_Pass()
{
    wrap_PassBase();
    wrap_ModulePass();
    wrap_ImmutablePass();
    wrap_FunctionPass();
    wrap_BasicBlockPass();
}

}} // namespace llvm::Python

// EOF
