/*
 * PassSupport.cpp: Python bindings for the LLVM compiler infrastructure
 * Copyright (c) 2004, 2005 Jarno Seppänen <firstname.lastname@iki.fi>.
 * Distributed under the Boost Software License, Version 1.0. (See
 * accompanying file LICENSE_1_0.txt or copy at
 * http://www.boost.org/LICENSE_1_0.txt)
 */

#include <boost/python.hpp>
#include <boost/noncopyable.hpp>
#include <llvm/Pass.h>
#include <llvm/Target/TargetMachine.h>

#include "scitbx/container_conversions.h"

using namespace boost::python;
using namespace scitbx::boost_python::container_conversions;

namespace llvm { namespace Python {

object getNormalCtor(const PassInfo& pi)
{
    // create callable object
    return make_function(pi.getNormalCtor(),
                         return_value_policy<manage_new_object>());
}

object getTargetCtor(const PassInfo& pi)
{
    // create callable object
    return make_function(pi.getTargetCtor(),
                         return_value_policy<manage_new_object>());
}

void wrap_PassInfo()
{
    scope PassInfoScope
        = class_<PassInfo, boost::noncopyable>
        ("PassInfo",
         init<const char*, const char*, const std::type_info&, unsigned char,
              optional<Pass*(*)(), Pass*(*)(TargetMachine&)> >())

        .def("getPassName", &PassInfo::getPassName)
        .def("setPassName", &PassInfo::setPassName)
        .def("getPassArgument", &PassInfo::getPassArgument)
        .def("getTypeInfo", &PassInfo::getTypeInfo,
             return_internal_reference<>())
        .def("getPassType", &PassInfo::getPassType)
        .def("getNormalCtor", getNormalCtor)
        .def("setNormalCtor", &PassInfo::setNormalCtor)
        .def("createPass", &PassInfo::createPass,
             return_value_policy<manage_new_object>())
        .def("getTargetCtor", getTargetCtor)
        .def("addInterfaceImplemented", &PassInfo::addInterfaceImplemented)
        .def("getInterfacesImplemented", &PassInfo::getInterfacesImplemented,
             return_value_policy<copy_const_reference>())
        ;

    // Register tuple conversion for vector<const PassInfo*>
    to_python_converter<std::vector<const PassInfo*>,
                        to_tuple<std::vector<const PassInfo*> > >();

    // FIXME Anonymous enum
    //PassInfoScope.attr("Analysis") = PassInfo::Analysis;
    //PassInfoScope.attr("Optimization") = PassInfo::Optimization;
    //PassInfoScope.attr("LLC") = PassInfo::LLC;
    //PassInfoScope.attr("AnalysisGroup") = PassInfo::AnalysisGroup;
}

void wrap_RegisterPassBase()
{
    class_<RegisterPassBase, boost::noncopyable>("RegisterPassBase")

        .def("getPassInfo", &RegisterPassBase::getPassInfo,
             return_internal_reference<>())
        ;
}

/*
void wrap_RegisterPass()
{
    // wrap RegisterPass, specialized for Python (TypeWrapper) object
    class_<RegisterPass<object>, boost::noncopyable, bases<RegisterPassBase> >
        ("RegisterPass",
         init<const char*, const char*,
              optional<unsigned char> >())

        .def(init<const char*, const char*, unsigned char, Pass*(*)()>())
        .def(init<const char*, const char*, unsigned char, Pass*(*)(TargetMachine&)>())
        ;
}

void wrap_RegisterOpt()
{
    class_<RegisterOpt<object>, boost::noncopyable, bases<RegisterPassBase> >
        ("RegisterOpt",
         init<const char*, const char*,
              optional<bool> >())

        .def(init<const char*, const char*, Pass*(*)(),
                 optional<bool> >())
        .def(init<const char*, const char*, FunctionPass*(*)(),
                 optional<bool> >())
        .def(init<const char*, const char*, Pass*(*)(TargetMachine&),
                 optional<bool> >())
        .def(init<const char*, const char*, FunctionPass*(*)(TargetMachine&),
                 optional<bool> >())
        ;
}
*/

//FIXME void wrap_RegisterAnalysis()
//FIXME void wrap_RegisterLLC()
//FIXME void wrap_RegisterAGBase()
//FIXME void wrap_RegisterAnalysisGroup()

// Allow virtual functiosn within Python
struct PassRegistrationListenerWrap
    : PassRegistrationListener, wrapper<PassRegistrationListener>
{
    virtual void passRegistered(const PassInfo* P)
    {
        this->get_override("passRegistered")(P);
    }
    virtual void passUnregistered(const PassInfo* P)
    {
        this->get_override("passUnregistered")(P);
    }
    virtual void passEnumerate(const PassInfo* P)
    {
        this->get_override("passEnumerate")(P);
    }
};

void wrap_PassRegistrationListener()
{
    class_<PassRegistrationListenerWrap, boost::noncopyable>("PassRegistrationListener")
        .def("passRegistered", pure_virtual(&PassRegistrationListener::passRegistered))
        .def("passUnregistered", pure_virtual(&PassRegistrationListener::passUnregistered))
        .def("enumeratePasses", &PassRegistrationListener::enumeratePasses)
        .def("passEnumerate", pure_virtual(&PassRegistrationListener::passEnumerate))
        ;
}

void wrap_PassSupport()
{
    wrap_PassInfo();
    wrap_RegisterPassBase();
//    wrap_RegisterPass();
//    wrap_RegisterOpt();
//    wrap_RegisterAnalysis();
//    wrap_RegisterLLC();
//    wrap_RegisterAGBase();
//    wrap_RegisterAnalysisGroup();
    wrap_PassRegistrationListener();
}

}} // namespace llvm::Python

// EOF
