/*
 * SymbolTable.cpp: Python bindings for the LLVM compiler infrastructure
 * Copyright (c) 2004, 2005 Jarno Seppänen <firstname.lastname@iki.fi>.
 * Distributed under the Boost Software License, Version 1.0. (See
 * accompanying file LICENSE_1_0.txt or copy at
 * http://www.boost.org/LICENSE_1_0.txt)
 */

#include <boost/python.hpp>
#include <boost/noncopyable.hpp>
#include <llvm/SymbolTable.h>
#include <llvm/Type.h>

#include <iostream>

using namespace boost::python;

namespace llvm { namespace Python {

// std::pair<const std::string, const Type*> ResultConverter
template <class T, class Base = default_call_policies>
struct StringType_pair_to_python : Base
{
    static bool convertible() {
        return true;
    }
    PyObject* operator()(std::pair<const std::string,Type*> const& x) const {
        object k(x.first);
        object v(ptr(x.second));
        object p(make_tuple(k, v));
        return incref(p.ptr());
    }
};

// std::pair<const std::string, const Value*> ResultConverter
template <class T, class Base = default_call_policies>
struct StringValue_pair_to_python : Base
{
    static bool convertible() {
        return true;
    }
    PyObject* operator()(std::pair<const std::string,Value*> const& x) const {
        object k(x.first);
        object v(ptr(x.second));
        object p(make_tuple(k, v));
        return incref(p.ptr());
    }
};

// std::pair<const std::string, const Type*> ResultConverterGenerator
struct return_string_Type_pair
{
    // metafunction returning the ResultConverter
    template <class T>
    struct apply {
        typedef StringType_pair_to_python<T> type;
    };
};

// std::pair<const std::string, const Value*> ResultConverterGenerator
struct return_string_Value_pair
{
    // metafunction returning the ResultConverter
    template <class T>
    struct apply {
        typedef StringValue_pair_to_python<T> type;
    };
};

std::pair<const std::string,Value*> test_pair()
{
    return make_pair(std::string("asdf"), (Value*)0);
}

void wrap_SymbolTable()
{
    def("test_pair", test_pair, return_value_policy<return_string_Value_pair>());

    // Name C++ overloads
    std::string (SymbolTable::*get_name1)(const Value*) const = &SymbolTable::get_name;
    std::string (SymbolTable::*get_name2)(const Type*) const = &SymbolTable::get_name;
    void (SymbolTable::*insert1)(Value*) = &SymbolTable::insert;
    void (SymbolTable::*insert2)(const std::string&, Value*) = &SymbolTable::insert;
    void (SymbolTable::*insert3)(const std::string&, const Type*) = &SymbolTable::insert;
    void (SymbolTable::*remove1)(Value*) = &SymbolTable::remove;
    void (SymbolTable::*remove2)(const Type*) = &SymbolTable::remove;
    Value* (SymbolTable::*remove3)(const std::string&, Value*) = &SymbolTable::remove;
    Type* (SymbolTable::*remove4)(SymbolTable::type_iterator) = &SymbolTable::remove;
    SymbolTable::plane_const_iterator (SymbolTable::*plane_begin)() const = &SymbolTable::plane_begin;
    SymbolTable::plane_const_iterator (SymbolTable::*plane_end)() const = &SymbolTable::plane_end;
    SymbolTable::value_const_iterator (SymbolTable::*value_begin)(const Type*) const = &SymbolTable::value_begin;
    SymbolTable::value_const_iterator (SymbolTable::*value_end)(const Type*) const = &SymbolTable::value_end;
    SymbolTable::type_const_iterator (SymbolTable::*type_begin)() const = &SymbolTable::type_begin;
    SymbolTable::type_const_iterator (SymbolTable::*type_end)() const = &SymbolTable::type_end;

    class_<SymbolTable, boost::noncopyable, bases<AbstractTypeUser> >("SymbolTable")

        .def("lookup", &SymbolTable::lookup,
             return_internal_reference<>())
        .def("lookupType", &SymbolTable::lookupType,
             return_internal_reference<>())
        .def("hasTypes", &SymbolTable::hasTypes)
        .def("isEmpty", &SymbolTable::isEmpty)
        .def("type_size", &SymbolTable::type_size)
        .def("num_types", &SymbolTable::num_types)
        .def("get_name", get_name1)
        .def("get_name", get_name2)
        .def("getUniqueName", &SymbolTable::getUniqueName)
        .def("dump", &SymbolTable::dump)
        .def("insert", insert1,
             with_custodian_and_ward<1, 2>()) // create Value* dependency
        .def("insert", insert2,
             with_custodian_and_ward<1, 3>()) // create Value* dependency
        .def("insert", insert3,
             with_custodian_and_ward<1, 3>()) // create Type* dependency
        //FIXME cannot destroy pointer dependencies?
        .def("remove", remove1)
        .def("remove", remove2)
        // the return values are NOT actually owned by SymbolTable, but
        // this assumes the lifetime dependency created during insert().
        .def("remove", remove3,
             return_internal_reference<>())
        // the return values are NOT actually owned by SymbolTable, but
        // this assumes the lifetime dependency created during insert().
        .def("remove", remove4,
             return_internal_reference<>())
        // the return values are NOT actually owned by SymbolTable, but
        // this assumes the lifetime dependency created during insert().
        .def("value_remove", &SymbolTable::value_remove,
             return_internal_reference<>())
        .def("strip", &SymbolTable::strip)
        .def("clear", &SymbolTable::clear)
        .add_property("planes", range(plane_begin, plane_end))
        //.def("plane_begin", plane_begin)
        //.def("plane_end", plane_end)
        //.def("values",
        //     range<return_value_policy<return_string_Value_pair> >
        //     (value_begin, value_end))
        //.def("values",
        //     range/*<return_value_policy<return_by_value> >*/(value_begin, value_end))
        .def("value_begin", value_begin)
        .def("value_end", value_end)
        .add_property("types",
                      range/*<return_value_policy<return_string_Type_pair>,
                             const std::pair<const std::string, const Type*> >*/
                          (type_begin, type_end))
        //.def("type_begin", type_begin)
        //.def("type_end", type_end)
        //.def("find", &SymbolTable::find)
        //.def("findPlane", &SymbolTable::findPlane) //deprecated
        ;
}

}} // namespace llvm::Python

// EOF
