/*
 * Path.cpp: Python bindings for the LLVM compiler infrastructure
 * Copyright (c) 2004, 2005 Jarno Seppänen <firstname.lastname@iki.fi>.
 * Distributed under the Boost Software License, Version 1.0. (See
 * accompanying file LICENSE_1_0.txt or copy at
 * http://www.boost.org/LICENSE_1_0.txt)
 */

#include <boost/python.hpp>
#include <boost/noncopyable.hpp>
#include <llvm/System/Path.h>

using namespace boost::python;

namespace llvm { namespace Python {

using namespace sys;

void wrap_System_Path(object module)
{
    scope SystemScope = module;

    enum_<LLVMFileType>("LLVMFileType")
        .value("UnknownFileType", UnknownFileType)
        .value("BytecodeFileType", BytecodeFileType)
        .value("CompressedBytecodeFileType", CompressedBytecodeFileType)
        .value("ArchiveFileType", ArchiveFileType)
        ;

    def("IdentifyFileType", IdentifyFileType);
    def("CopyFile", CopyFile);

    scope PathScope
        = class_<Path, boost::noncopyable>("Path")
        .def(init<const std::string>())

        .def("GetRootDirectory", &Path::GetRootDirectory)
        .staticmethod("GetRootDirectory")
        .def("GetTemporaryDirectory", &Path::GetTemporaryDirectory)
        .staticmethod("GetTemporaryDirectory")
        .def("GetSystemLibraryPaths", &Path::GetSystemLibraryPaths)
        .staticmethod("GetSystemLibraryPaths")
        .def("GetBytecodeLibraryPaths", &Path::GetBytecodeLibraryPaths)
        .staticmethod("GetBytecodeLibraryPaths")
        .def("FindLibrary", &Path::FindLibrary)
        .staticmethod("FindLibrary")
        .def("GetLLVMDefaultConfigDir", &Path::GetLLVMDefaultConfigDir)
        .staticmethod("GetLLVMDefaultConfigDir")
        .def("GetLLVMConfigDir", &Path::GetLLVMConfigDir)
        .staticmethod("GetLLVMConfigDir")
        .def("GetUserHomeDirectory", &Path::GetUserHomeDirectory)
        .staticmethod("GetUserHomeDirectory")
        .def("GetDLLSuffix", &Path::GetDLLSuffix)
        .staticmethod("GetDLLSuffix")
        //operator= doesn't exist in Python
        .def(self == self) // __eq__
        .def(self != self) // __ne__
        .def(self < self) // __lt__
        .def("isValid", &Path::isValid)
        .def("isEmpty", &Path::isEmpty)
        .def("isFile", &Path::isFile)
        .def("isDirectory", &Path::isDirectory)
        //FIXME llvm.so: undefined symbol: _ZNK4llvm3sys4Path15isRootDirectoryEv
        //.def("isRootDirectory", &Path::isRootDirectory)
        .def("hasMagicNumber", &Path::hasMagicNumber)
        .def("getMagicNumber", &Path::getMagicNumber)
        .def("isArchive", &Path::isArchive)
        .def("isBytecodeFile", &Path::isBytecodeFile)
        .def("isDynamicLibrary", &Path::isDynamicLibrary)
        .def("exists", &Path::exists)
        .def("readable", &Path::readable)
        .def("writable", &Path::writable)
        .def("executable", &Path::executable)
        .def("toString", &Path::toString,
             return_value_policy<return_by_value>())
        .def("getLast", &Path::getLast)
        .def("getBasename", &Path::getBasename)
        .def("getDirectoryContents", &Path::getDirectoryContents)
        .def("destroyDirectory", &Path::destroyDirectory)
        .def("destroyFile", &Path::destroyFile)
        //.def("c_str", &Path::c_str,
        //     return_internal_reference<>())
        .def("clear", &Path::clear)
        .def("getStatusInfo", &Path::getStatusInfo)
        .def("getTimestamp", &Path::getTimestamp)
        .def("getSize", &Path::getSize)
        .def("makeReadable", &Path::makeReadable)
        .def("makeWriteable", &Path::makeWriteable)
        .def("makeExecutable", &Path::makeExecutable)
        .def("setDirectory", &Path::setDirectory)
        .def("setFile", &Path::setFile)
        .def("appendDirectory", &Path::appendDirectory)
        .def("elideDirectory", &Path::elideDirectory)
        .def("appendFile", &Path::appendFile)
        .def("elideFile", &Path::elideFile)
        .def("appendSuffix", &Path::appendSuffix)
        .def("elideSuffix", &Path::elideSuffix)
        .def("makeUnique", &Path::makeUnique)
        .def("createDirectory", &Path::createDirectory)
        .def("createFile", &Path::createFile)
        .def("createTemporaryFile", &Path::createTemporaryFile)
        .def("renameFile", &Path::renameFile)
        .def("setStatusInfo", &Path::setStatusInfo)
        ;

    // Nested within PathScope
    class_<Path::StatusInfo>("StatusInfo")
        .def_readwrite("fileSize", &Path::StatusInfo::fileSize)
        .def_readwrite("modTime", &Path::StatusInfo::modTime)
        .def_readwrite("mode", &Path::StatusInfo::mode)
        .def_readwrite("user", &Path::StatusInfo::user)
        .def_readwrite("group", &Path::StatusInfo::group)
        .def_readwrite("isDir", &Path::StatusInfo::isDir)
        ;
}

}} // namespace llvm::Python

// EOF
